#ifndef __BLEND2CS_ERROR_H__
#define __BLEND2CS_ERROR_H__

#include <exception>

namespace dna
{

/**
 * This is the main class used for exception error handling. You can give a
 * printf format string when constructing an Error and later read the message.
 */ 
class Error : public std::exception
{
public:
    /** Construct a new error message. You can use a printf format string and
     * arguments for constructing more complicated error messages.
     */
    Error (const char* msg, ...) throw();
    /// copy constructor
    Error (const Error& error) throw();
    /// destructor
    virtual ~Error() throw();

    /** Returns the error message */
    const char* what () const throw();

private:
    /// holds the error message and should be freed with delete[] later.
    char* message;
};

} // end of namespace dna

#endif

