#include <config.h>

#include "variable.h"
#include "error.h"

#include "structure.h"

namespace dna
{

BLStruct* Structure::INVALID_PTR = (BLStruct*) (~0x0);

Structure::Structure(BLStruct* blstruct)
{
    this->blstruct = blstruct;
}

Structure::~Structure()
{
}

Variable Structure::getVariable(const char* path) const
{
    BLData* data = BLFindData(blstruct, path);
    if (!data)
	throw Error ("Structure::getVariable: couldn't find variable '%s'"
		" in Structure '%s'.", path, blstruct->struct_name);

    return Variable(data);
}

const char* Structure::getName() const
{
    return blstruct->struct_name;
}

bool Structure::isGood() const
{
    return (blstruct != 0) && (blstruct != INVALID_PTR);
}

Structure::Iterator Structure::getIterator(const char* structname) const
{
    return Iterator(blstruct, structname);
}

Structure::Iterator::Iterator()
{
    structname = 0;
    blstruct = 0;
}

Structure::Iterator::Iterator(BLStruct* newblstruct, const char* newstructname)
{
    blstruct = newblstruct;
    structname = newstructname;
    
    if (blstruct == INVALID_PTR)
	blstruct = 0;

    if (structname) {
	blstruct = BLFindStruct(blstruct, structname);
    }
}

bool Structure::Iterator::hasNext()
{
    return blstruct != 0;
}

Structure Structure::Iterator::next()
{
    if (structname) {
	BLStruct* result = blstruct;
	blstruct = BLFindStruct(blstruct->next, structname);
	if (blstruct == INVALID_PTR)
	    blstruct = 0;
	return Structure(result);
    } else {
	BLStruct* result = blstruct;
	blstruct = blstruct->next;
	if (blstruct == INVALID_PTR)
	    blstruct = 0;
	return Structure(result);
    }
}

Structure Structure::findSubsequentStructure(const char* newstructname)
{
  // This routine finds the next structure named newstructname that
  // occurs before the next structure with the same name as this one.
  BLStruct *mStruct;
  for (mStruct=blstruct->next; mStruct!=NULL ;mStruct=mStruct->next) {
    //    fprintf(stderr,"-> %s ", mStruct->struct_name);
    if (!strcmp(mStruct->struct_name,newstructname)) return Structure(mStruct);
    if (!strcmp(mStruct->struct_name,blstruct->struct_name)) break;
  }
  return Structure(0);
}

Structure::ID Structure::getID() const
{
    return (ID)blstruct;
}

bool Structure::operator < (const Structure& other) const
{
    return getID() < other.getID();
}

bool Structure::operator == (const Structure& other) const
{
    return getID() == other.getID();
}

} // end of namespace dna

