#ifndef __BLEND2CS_STRUCTURE_H__
#define __BLEND2CS_STRUCTURE_H__

#include <stdio.h>
#include "dna/btypes.h"
#include "dna/bfile.h"
#include "dna/bdata.h"
#include "dna/bparse.h"

namespace dna
{

class Variable;
    
/**
 * This class is a c++ wrapper around the BLStruct structure
 */
class Structure
{
public:
    Structure(BLStruct* blstruct);
    ~Structure();

    static BLStruct* INVALID_PTR;

    /** returns the variable at the path in the structure */
    Variable getVariable(const char* path) const;
    /** returns the type of the structure */
    const char* getName() const;
    /** returns true if the Structure points not to zero or an invalid ptr */
    bool isGood() const;

    // iterators
    class Iterator
    {
    public:
	Iterator();
	
	bool hasNext();
	Structure next();

	void operator == (const Iterator& other)
	{ blstruct = other.blstruct; structname = other.structname; }
    
    protected:
	friend class Structure;
	Iterator(BLStruct* blstruct, const char* structname = 0);

    private:
	BLStruct* blstruct;
	const char* structname;
    };

    /** Returns an iterator over the structurelist.
     * If structurename != 0, then the iterator will only return structures with
     * the given name
     */
    Iterator getIterator (const char* structname = 0) const;
    Structure findSubsequentStructure(const char* newstructname);
    /** returns a unique ID for the Structure */
    typedef void* ID;
    ID getID() const;

    /** compare Structures by IDs */
    bool operator < (const Structure& other) const;
    bool operator == (const Structure& other) const;
    operator bool() { return (bool) blstruct; }

private:
    BLStruct* blstruct;
};

} // end of namespace dna

#endif

