#ifndef __BLEND2CS_BOX2D_H__
#define __BLEND2CS_BOX2D_H__

#include <string>
#include "vector2d.h"

namespace blend2cs
{

class Box2D
{
public:
    Box2D(const Vector2D& mi, const Vector2D& ma)
    {
        minbox = mi;
        maxbox = ma;
    }
    Box2D(float minx=1000000000.0f, float miny=1000000000.0f,
    	float maxx=-1000000000.0f, float maxy=-1000000000.0f)
    {
        set(minx, miny, maxx, maxy);
    }
    ~Box2D() { }

    void set(const Vector2D& mi, const Vector2D& ma)
    {
        minbox = mi;
        maxbox = ma;
    }

    void set(float newminx, float newminy, float newmaxx, float newmaxy)
    {
	minbox.setX(newminx);
	minbox.setY(newminy);
	maxbox.setX(newmaxx);
	maxbox.setY(newmaxy);
    }

    void setMin(const Vector2D& m)
    { minbox = m; }
    void setMax(const Vector2D& m)
    { maxbox = m; }

    void setMinX(float newminx)
    { minbox.setX(newminx); }
    void setMinY(float newminy)
    { minbox.setY(newminy); }
    void setMaxX(float newmaxx)
    { maxbox.setX(newmaxx); }
    void setMaxY(float newmaxy)
    { maxbox.setY(newmaxy); }

    const Vector2D& getMin() const
    { return minbox; }
    const Vector2D& getMax() const
    { return maxbox; }

    bool operator == (const Box2D& other) const
    { return minbox == other.minbox && maxbox == other.maxbox; }

    bool operator != (const Box2D& other) const
    { return ! (*this == other); }

    void addBoundingVertex(float x, float y) {
        if (x < minbox.getX()) minbox.setX(x);
	if (x > maxbox.getX()) maxbox.setX(x);
        if (y < minbox.getY()) minbox.setY(y);
	if (y > maxbox.getY()) maxbox.setY(y);
    }
    void addBoundingVertex(const Vector2D& v) {
        addBoundingVertex(v.getX(), v.getY());
    }

    bool in(const Vector2D& v) const {
        return v.getX() >= minbox.getX() && v.getX() <= maxbox.getX()
		&& v.getY() >= minbox.getY() && v.getY() <= maxbox.getY();
    }

private:
    Vector2D minbox;
    Vector2D maxbox;
};

}

#endif

