#include <config.h>

#include <math.h>

#include "matrix3d.h"
#include "xmlwriter.h"

namespace blend2cs
{

Matrix3D::Matrix3D(float m11, float m12, float m13,
		   float m21, float m22, float m23,
		   float m31, float m32, float m33)
{
    matrix[0][0]=m11;
    matrix[0][1]=m12;
    matrix[0][2]=m13;
    matrix[1][0]=m21;
    matrix[1][1]=m22;
    matrix[1][2]=m23;
    matrix[2][0]=m31;
    matrix[2][1]=m32;
    matrix[2][2]=m33;
}

Matrix3D::~Matrix3D()
{
}

void Matrix3D::calcRotation(float rotx, float roty, float rotz)
{
    matrix[0][0]=cos(roty)*cos(rotz);
    matrix[0][1]=(sin(rotx)*sin(roty)*cos(rotz))-(cos(rotx)*sin(rotz));
    matrix[0][2]=(cos(rotx)*sin(roty)*cos(rotz))+(sin(rotx)*sin(rotz));
                                                                                
    matrix[2][0]=cos(roty)*sin(rotz);
    matrix[2][1]=(sin(rotx)*sin(roty)*sin(rotz))+(cos(rotx)*cos(rotz));
    matrix[2][2]=(cos(rotx)*sin(roty)*sin(rotz))-(sin(rotx)*cos(rotz));
                                                                                
    matrix[1][0]=-sin(roty);
    matrix[1][1]=sin(rotx)*cos(roty);
    matrix[1][2]=cos(rotx)*cos(roty);
}

void Matrix3D::write(XmlWriter& xml) const
{
    xml.writeTag("m11");
    xml.write(Get(0,0));
    xml.writeTag("m12");
    xml.write(Get(0,1));
    xml.writeTag("m13");
    xml.write(Get(0,2));
    xml.writeTag("m21");
    xml.write(Get(1,0));
    xml.writeTag("m22");
    xml.write(Get(1,1));
    xml.writeTag("m23");
    xml.write(Get(1,2));
    xml.writeTag("m31");
    xml.write(Get(2,0));
    xml.writeTag("m32");
    xml.write(Get(2,1));
    xml.writeTag("m33");
    xml.write(Get(2,2));
}

} // end of namespace blend2cs

