#ifndef __BLEND2CS_MATRIX3D_H__
#define __BLEND2CS_MATRIX3D_H__

#include "vector3d.h"

namespace blend2cs
{

class XmlWriter;

class Matrix3D
{
public:
    Matrix3D(float m11=1, float m12=0, float m13=0,
	     float m21=0, float m22=1, float m23=0,
	     float m31=0, float m32=0, float m33=1);
    ~Matrix3D();

    /** this routine calculates the rotation coefficients for an object;
     * This rotation includes a reflection about the y-z axis, so it is
     * not suitable for direct consumption by CS.
     */
    void calcRotation(float x, float y, float z);

    float Get(int row, int col) const
    { return matrix[row][col]; }

    inline friend Vector3D operator* (const Matrix3D& m, const Vector3D& v) {
        return Vector3D (
		m.matrix[0][0]*v.getX() + m.matrix[0][1]*v.getY() + m.matrix[0][2]*v.getZ(),
                m.matrix[1][0]*v.getX() + m.matrix[1][1]*v.getY() + m.matrix[1][2]*v.getZ(),
                m.matrix[2][0]*v.getX() + m.matrix[2][1]*v.getY() + m.matrix[2][2]*v.getZ());
    }

    void write(XmlWriter& xml) const;

    void dump() {
        printf ("%g,%g,%g\n%g,%g,%g\n%g,%g,%g\n",
    	    GetRnd(0,0), GetRnd(0,1), GetRnd(0,2),
    	    GetRnd(1,0), GetRnd(1,1), GetRnd(1,2),
    	    GetRnd(2,0), GetRnd(2,1), GetRnd(2,2)); fflush (stdout);
    }


private:
    float matrix[3][3];

    float GetRnd(int row, int col) const
    { return float (int (matrix[row][col] * 10000)) / 10000.0f; }
};

} // end of namespace blend2cs

#endif
