#ifndef __BLEND2CS_MATRIX4D_H__
#define __BLEND2CS_MATRIX4D_H__

namespace blend2cs
{

class Matrix4D
{
public:
    Matrix4D(float m00=1, float m01=0, float m02=0, float m03=0,
	     float m10=0, float m11=1, float m12=0, float m13=0,
	     float m20=0, float m21=0, float m22=1, float m23=0,
	     float m30=0, float m31=0, float m32=0, float m33=1) {
        m[0][0] = m00;
        m[0][1] = m01;
        m[0][2] = m02;
        m[0][3] = m03;
        m[1][0] = m10;
        m[1][1] = m11;
        m[1][2] = m12;
        m[1][3] = m13;
        m[2][0] = m20;
        m[2][1] = m21;
        m[2][2] = m22;
        m[2][3] = m23;
        m[3][0] = m30;
        m[3][1] = m31;
        m[3][2] = m32;
        m[3][3] = m33;
    }
    ~Matrix4D() { }

    double Determinant() {
        return 
	    m[0][3] * m[1][2] * m[2][1] * m[3][0]-m[0][2] * m[1][3] * m[2][1] * m[3][0]-m[0][3] * m[1][1] * m[2][2] * m[3][0]+m[0][1] * m[1][3] * m[2][2] * m[3][0]+
	    m[0][2] * m[1][1] * m[2][3] * m[3][0]-m[0][1] * m[1][2] * m[2][3] * m[3][0]-m[0][3] * m[1][2] * m[2][0] * m[3][1]+m[0][2] * m[1][3] * m[2][0] * m[3][1]+
	    m[0][3] * m[1][0] * m[2][2] * m[3][1]-m[0][0] * m[1][3] * m[2][2] * m[3][1]-m[0][2] * m[1][0] * m[2][3] * m[3][1]+m[0][0] * m[1][2] * m[2][3] * m[3][1]+
	    m[0][3] * m[1][1] * m[2][0] * m[3][2]-m[0][1] * m[1][3] * m[2][0] * m[3][2]-m[0][3] * m[1][0] * m[2][1] * m[3][2]+m[0][0] * m[1][3] * m[2][1] * m[3][2]+
	    m[0][1] * m[1][0] * m[2][3] * m[3][2]-m[0][0] * m[1][1] * m[2][3] * m[3][2]-m[0][2] * m[1][1] * m[2][0] * m[3][3]+m[0][1] * m[1][2] * m[2][0] * m[3][3]+
	    m[0][2] * m[1][0] * m[2][1] * m[3][3]-m[0][0] * m[1][2] * m[2][1] * m[3][3]-m[0][1] * m[1][0] * m[2][2] * m[3][3]+m[0][0] * m[1][1] * m[2][2] * m[3][3];
    }

    //friend Matrix4D operator* (const Matrix4D& a1, const Matrix4D& m2);

    inline friend Matrix4D operator* (const Matrix4D& a1, const Matrix4D& a2) {
	Matrix4D m;
        m.m[0][0] = a1.m[0][0]*a2.m[0][0] + a1.m[0][1]*a2.m[1][0] + a1.m[0][2]*a2.m[2][0] + a1.m[0][3]*a2.m[3][0];
        m.m[0][1] = a1.m[0][0]*a2.m[0][1] + a1.m[0][1]*a2.m[1][1] + a1.m[0][2]*a2.m[2][1] + a1.m[0][3]*a2.m[3][1];
        m.m[0][2] = a1.m[0][0]*a2.m[0][2] + a1.m[0][1]*a2.m[1][2] + a1.m[0][2]*a2.m[2][2] + a1.m[0][3]*a2.m[3][2];
        m.m[0][3] = a1.m[0][0]*a2.m[0][3] + a1.m[0][1]*a2.m[1][3] + a1.m[0][2]*a2.m[2][3] + a1.m[0][3]*a2.m[3][3];
        m.m[1][0] = a1.m[1][0]*a2.m[0][0] + a1.m[1][1]*a2.m[1][0] + a1.m[1][2]*a2.m[2][0] + a1.m[1][3]*a2.m[3][0];
        m.m[1][1] = a1.m[1][0]*a2.m[0][1] + a1.m[1][1]*a2.m[1][1] + a1.m[1][2]*a2.m[2][1] + a1.m[1][3]*a2.m[3][1];
        m.m[1][2] = a1.m[1][0]*a2.m[0][2] + a1.m[1][1]*a2.m[1][2] + a1.m[1][2]*a2.m[2][2] + a1.m[1][3]*a2.m[3][2];
        m.m[1][3] = a1.m[1][0]*a2.m[0][3] + a1.m[1][1]*a2.m[1][3] + a1.m[1][2]*a2.m[2][3] + a1.m[1][3]*a2.m[3][3];
        m.m[2][0] = a1.m[2][0]*a2.m[0][0] + a1.m[2][1]*a2.m[1][0] + a1.m[2][2]*a2.m[2][0] + a1.m[2][3]*a2.m[3][0];
        m.m[2][1] = a1.m[2][0]*a2.m[0][1] + a1.m[2][1]*a2.m[1][1] + a1.m[2][2]*a2.m[2][1] + a1.m[2][3]*a2.m[3][1];
        m.m[2][2] = a1.m[2][0]*a2.m[0][2] + a1.m[2][1]*a2.m[1][2] + a1.m[2][2]*a2.m[2][2] + a1.m[2][3]*a2.m[3][2];
        m.m[2][3] = a1.m[2][0]*a2.m[0][3] + a1.m[2][1]*a2.m[1][3] + a1.m[2][2]*a2.m[2][3] + a1.m[2][3]*a2.m[3][3];
        m.m[3][0] = a1.m[3][0]*a2.m[0][0] + a1.m[3][1]*a2.m[1][0] + a1.m[3][2]*a2.m[2][0] + a1.m[3][3]*a2.m[3][0];
        m.m[3][1] = a1.m[3][0]*a2.m[0][1] + a1.m[3][1]*a2.m[1][1] + a1.m[3][2]*a2.m[2][1] + a1.m[3][3]*a2.m[3][1];
        m.m[3][2] = a1.m[3][0]*a2.m[0][2] + a1.m[3][1]*a2.m[1][2] + a1.m[3][2]*a2.m[2][2] + a1.m[3][3]*a2.m[3][2];
        m.m[3][3] = a1.m[3][0]*a2.m[0][3] + a1.m[3][1]*a2.m[1][3] + a1.m[3][2]*a2.m[2][3] + a1.m[3][3]*a2.m[3][3];
	return m;
    } 

    void Invert() {
        double det = 1.0 / Determinant();
        double t00 = m[1][2]*m[2][3]*m[3][1] - m[1][3]*m[2][2]*m[3][1] + m[1][3]*m[2][1]*m[3][2] - m[1][1]*m[2][3]*m[3][2] - m[1][2]*m[2][1]*m[3][3] + m[1][1]*m[2][2]*m[3][3];
        double t01 = m[0][3]*m[2][2]*m[3][1] - m[0][2]*m[2][3]*m[3][1] - m[0][3]*m[2][1]*m[3][2] + m[0][1]*m[2][3]*m[3][2] + m[0][2]*m[2][1]*m[3][3] - m[0][1]*m[2][2]*m[3][3];
        double t02 = m[0][2]*m[1][3]*m[3][1] - m[0][3]*m[1][2]*m[3][1] + m[0][3]*m[1][1]*m[3][2] - m[0][1]*m[1][3]*m[3][2] - m[0][2]*m[1][1]*m[3][3] + m[0][1]*m[1][2]*m[3][3];
        double t03 = m[0][3]*m[1][2]*m[2][1] - m[0][2]*m[1][3]*m[2][1] - m[0][3]*m[1][1]*m[2][2] + m[0][1]*m[1][3]*m[2][2] + m[0][2]*m[1][1]*m[2][3] - m[0][1]*m[1][2]*m[2][3];
        double t10 = m[1][3]*m[2][2]*m[3][0] - m[1][2]*m[2][3]*m[3][0] - m[1][3]*m[2][0]*m[3][2] + m[1][0]*m[2][3]*m[3][2] + m[1][2]*m[2][0]*m[3][3] - m[1][0]*m[2][2]*m[3][3];
        double t11 = m[0][2]*m[2][3]*m[3][0] - m[0][3]*m[2][2]*m[3][0] + m[0][3]*m[2][0]*m[3][2] - m[0][0]*m[2][3]*m[3][2] - m[0][2]*m[2][0]*m[3][3] + m[0][0]*m[2][2]*m[3][3];
        double t12 = m[0][3]*m[1][2]*m[3][0] - m[0][2]*m[1][3]*m[3][0] - m[0][3]*m[1][0]*m[3][2] + m[0][0]*m[1][3]*m[3][2] + m[0][2]*m[1][0]*m[3][3] - m[0][0]*m[1][2]*m[3][3];
        double t13 = m[0][2]*m[1][3]*m[2][0] - m[0][3]*m[1][2]*m[2][0] + m[0][3]*m[1][0]*m[2][2] - m[0][0]*m[1][3]*m[2][2] - m[0][2]*m[1][0]*m[2][3] + m[0][0]*m[1][2]*m[2][3];
        double t20 = m[1][1]*m[2][3]*m[3][0] - m[1][3]*m[2][1]*m[3][0] + m[1][3]*m[2][0]*m[3][1] - m[1][0]*m[2][3]*m[3][1] - m[1][1]*m[2][0]*m[3][3] + m[1][0]*m[2][1]*m[3][3];
        double t21 = m[0][3]*m[2][1]*m[3][0] - m[0][1]*m[2][3]*m[3][0] - m[0][3]*m[2][0]*m[3][1] + m[0][0]*m[2][3]*m[3][1] + m[0][1]*m[2][0]*m[3][3] - m[0][0]*m[2][1]*m[3][3];
        double t22 = m[0][1]*m[1][3]*m[3][0] - m[0][3]*m[1][1]*m[3][0] + m[0][3]*m[1][0]*m[3][1] - m[0][0]*m[1][3]*m[3][1] - m[0][1]*m[1][0]*m[3][3] + m[0][0]*m[1][1]*m[3][3];
        double t23 = m[0][3]*m[1][1]*m[2][0] - m[0][1]*m[1][3]*m[2][0] - m[0][3]*m[1][0]*m[2][1] + m[0][0]*m[1][3]*m[2][1] + m[0][1]*m[1][0]*m[2][3] - m[0][0]*m[1][1]*m[2][3];
        double t30 = m[1][2]*m[2][1]*m[3][0] - m[1][1]*m[2][2]*m[3][0] - m[1][2]*m[2][0]*m[3][1] + m[1][0]*m[2][2]*m[3][1] + m[1][1]*m[2][0]*m[3][2] - m[1][0]*m[2][1]*m[3][2];
        double t31 = m[0][1]*m[2][2]*m[3][0] - m[0][2]*m[2][1]*m[3][0] + m[0][2]*m[2][0]*m[3][1] - m[0][0]*m[2][2]*m[3][1] - m[0][1]*m[2][0]*m[3][2] + m[0][0]*m[2][1]*m[3][2];
        double t32 = m[0][2]*m[1][1]*m[3][0] - m[0][1]*m[1][2]*m[3][0] - m[0][2]*m[1][0]*m[3][1] + m[0][0]*m[1][2]*m[3][1] + m[0][1]*m[1][0]*m[3][2] - m[0][0]*m[1][1]*m[3][2];
        double t33 = m[0][1]*m[1][2]*m[2][0] - m[0][2]*m[1][1]*m[2][0] + m[0][2]*m[1][0]*m[2][1] - m[0][0]*m[1][2]*m[2][1] - m[0][1]*m[1][0]*m[2][2] + m[0][0]*m[1][1]*m[2][2];
        m[0][0] = t00 * det;
        m[0][1] = t01 * det;
        m[0][2] = t02 * det;
        m[0][3] = t03 * det;
        m[1][0] = t10 * det;
        m[1][1] = t11 * det;
        m[1][2] = t12 * det;
        m[1][3] = t13 * det;
        m[2][0] = t20 * det;
        m[2][1] = t21 * det;
        m[2][2] = t22 * det;
        m[2][3] = t23 * det;
        m[3][0] = t30 * det;
        m[3][1] = t31 * det;
        m[3][2] = t32 * det;
        m[3][3] = t33 * det;
    }

    float Get(int row, int col) const
    { return m[row][col]; }

    void dump() {
        printf ("%g,%g,%g,%g\n%g,%g,%g,%g\n%g,%g,%g,%g\n%g,%g,%g,%g\n",
    	    GetRnd(0,0), GetRnd(0,1), GetRnd(0,2), GetRnd(0,3),
    	    GetRnd(1,0), GetRnd(1,1), GetRnd(1,2), GetRnd(1,3),
    	    GetRnd(2,0), GetRnd(2,1), GetRnd(2,2), GetRnd(2,3),
    	    GetRnd(3,0), GetRnd(3,1), GetRnd(3,2), GetRnd(3,3)); fflush (stdout);
    }

private:
    float m[4][4];

    float GetRnd(int row, int col) const
    { return float (int (m[row][col] * 10000)) / 10000.0f; }
};

} // end of namespace blend2cs

#endif
