#ifndef __MESHFLAGS_H__
#define __MESHFLAGS_H__

#include "vector3d.h"
#include "polygonflags.h"

namespace blend2cs
{

class TerrainMeshFlags
{
public:
    TerrainMeshFlags ()
    	: resolution(256),
	  saveheightmap(true),
	  savematerialmap(true),
	  savebasematerial(true),
	  splatting_distance(200), block_resolution(16),
	  block_split_distance(8), minimum_block_size(32),
	  cd_resolution(256), cd_lod_cost(-1),
	  lightmap_resolution(257), staticlighting(true),
	  castshadows(true)
    {
        heightmap = "heightmap.png";
        materialmap = "materialmap.png";
	basematerial_name = "basematerial";
	basematerial = "basematerial.png";
    }

    bool operator == (const TerrainMeshFlags& o) const
    {
        return heightmap == o.heightmap
		&& materialmap == o.materialmap
		&& basematerial_name == o.basematerial_name
		&& basematerial == o.basematerial
		&& resolution == o.resolution
		&& saveheightmap == o.saveheightmap
		&& savematerialmap == o.savematerialmap
		&& savebasematerial == o.savebasematerial
		&& splatting_distance == o.splatting_distance
		&& block_split_distance == o.block_split_distance
		&& minimum_block_size == o.minimum_block_size
		&& cd_resolution == o.cd_resolution
		&& cd_lod_cost == o.cd_lod_cost
		&& lightmap_resolution == o.lightmap_resolution
		&& staticlighting == o.staticlighting
		&& castshadows == o.castshadows;
    }

    std::string heightmap;	// For the add-on.
    std::string materialmap;	// For the meshobj.
    std::string basematerial_name;// For the meshobj.
    std::string basematerial;	// For the meshobj.
    int resolution;		// For the generation.
    bool saveheightmap;		// For the generation.
    bool savematerialmap;	// For the generation.
    bool savebasematerial;	// For the generation.
    float splatting_distance;	// For the meshobj.
    int block_resolution;	// For the meshobj.
    float block_split_distance;	// For the meshobj.
    float minimum_block_size;	// For the meshobj.
    int cd_resolution;		// For the meshobj.
    float cd_lod_cost;		// For the meshobj.
    int lightmap_resolution;	// For the meshobj.
    bool staticlighting;	// For the meshobj.
    bool castshadows;		// For the meshobj.
};

class MeshFlags
{
public:
    MeshFlags()
        : type(INVALID), localshadows(false), smoothing(false),
	  colldet(true), viscull(true), exported(true),
	  noclip(false),
	  mixmode(COPY), alpha(0)
    {
    }
    
    bool operator == (const MeshFlags& o) const
    {
        return scale == o.scale && polygonflags == o.polygonflags
            && localshadows == o.localshadows && smoothing == o.smoothing
	    && colldet == o.colldet && viscull == o.viscull
	    && exported == o.exported && noclip == o.noclip
	    && mixmode == o.mixmode
	    && alpha == o.alpha && terrain_flags == o.terrain_flags;
    }

    enum Type
    { TRIANGLES, THING, INVALID };
    enum MixMode
    { COPY, ADD, MULTIPLY, MULTIPLY2, ALPHA };
    
    Type type;
    Vector3D scale;
    PolygonFlags polygonflags;
    bool localshadows;
    bool smoothing;
    bool colldet;	// Enable/disable collision detection for entire mesh.
    bool viscull;	// Enable/disable visibility culling for entire mesh.
    bool exported;	// Enable/disable export of this object. Default true.
    bool noclip;	// CS mesh object noclip option.

    MixMode mixmode;
    float alpha;

    TerrainMeshFlags terrain_flags;
};

} // end of namespace blend2cs

#endif

