#ifndef __BLEND2CS_POLYGON_H__
#define __BLEND2CS_POLYGON_H__

#include <string>
#include <dna++/error.h>
#include <dna++/blendertypes.h>
#include <dna++/variable.h>
#include <dna++/structure.h>

#include "object.h"
#include "vector3d.h"
#include "polygonflags.h"

namespace blend2cs
{

class XmlWriter;

class Polygon
{
public:
    Polygon();
    Polygon(const Polygon& other);
    // Parsing constructor:
    Polygon(dna::Structure mface, dna::Structure tface, 
	    const PolygonFlags& flags, int polyn);
    ~Polygon();

    void clearPoly();

    /** Creates a polygon with flipped normal */
    Polygon* createInvertedPolygon() const;

    void addVertex(int num)
    {
	if (vertexcount>=4)
	    throw dna::Error("Polygon::addVertex: only 4 vertices supported.");
	vertices[vertexcount++] = num;
    }
    int getVertexCount() const
    { return vertexcount; }
    int getVertex(int num) const
    { 
	if (num<0 || num>=vertexcount)
	    throw dna::Error("Polygon::getVertex: vertex index out of bounds.");
	return vertices[num];
    }
    
    void setUV(int idx, float u, float v)
    {
	if (idx<0 || idx>3)
	    throw dna::Error("Polygon::setUV: vertex index out of bounds.");
	uv[idx][0] = u;
	uv[idx][1] = v;
    }
    void fixUV();
    
    float getU(int idx) const
    {
	if (idx<0 || idx>3)
	    throw dna::Error("Polygon::getU: vertex index out of bounds.");
	return uv[idx][0];
    }
    float getV(int idx) const
    {
	if (idx<0 || idx>3)
	    throw dna::Error("Polygon::getV: vertex index out of bounds.");
	return uv[idx][1];
    }

    const std::string& getMaterial() const
    { return material; }
    void setMaterial(const std::string& name)
    { material = name; }

    PolygonFlags& getFlags()
    { return flags; }
    const PolygonFlags& getFlags() const
    { return flags; }

    void write(XmlWriter& writer) const;
	
private:
    int vertexcount;
    int vertices[4];
    float uv[4][2];

    std::string material;

    // flags used in CS
    PolygonFlags flags;
};

} // end of namespace blend2cs

#endif
