#include <config.h>

#include <iostream>
#include <dna++/error.h>
#include <dna++/variable.h>
#include <dna++/blendertypes.h>

#include "property.h"

namespace blend2cs
{

Property::Property(dna::Structure newstructure)
    : structure(newstructure)
{
    float_value = 0;
    int_value = 0;
    parseProperty();
}

Property::~Property()
{
}

void Property::parseProperty()
{
    name = structure.getVariable("name").getValueString();
    
    // convert structure value to string
    // convert structure value to stringr str[256];
    char str[256];
    switch (structure.getVariable("type").getValueShort()) {
	case PROP_BOOL:
	    value = structure.getVariable("data").getValueInt()
		    ? "true" : "false";
	    break;
	case PROP_INT:
    	    int_value = structure.getVariable("data").getValueInt();
	    snprintf (str, 256, "%d", int_value);
	    value = str;
	    break;
	case PROP_TIME:
	case PROP_FLOAT:
	{
    	    int data = structure.getVariable("data").getValueInt();
	    float_value = * ((float*) (&data));
	    snprintf(str, 256, "%f", float_value);
	    value = str;
	    break;
	}
	case PROP_STRING:
	{
	    dna::Structure strstruct 
		= structure.getVariable("poin").getValueStructure();
            const char* str =
                strstruct.getVariable("next").getValueInternalString();
            if(str)
                value = str;
	    break;
	}
	default:
      	    std::cerr << "Unsupported type at property '"
		<< structure.getVariable("name").getValueString() 
		<< "'" << std::endl;
	    break;
    }
}

bool Property::getValueBool() const
{
    if (value=="true" || value=="on" || value=="yes" || value=="enabled")
	return true;
    if (value=="false" || value=="off" || value=="no" || value=="disabled")
	return false;

    throw dna::Error("couldn't parse boolean value.");
}

} // end of namespace blend2cs
