#include <config.h>

#include <iostream>
#include <dna++/error.h>
#include <dna++/variable.h>
#include <dna++/listiterator.h>

#include "sectorsettings.h"

namespace blend2cs
{

SectorSettings::SectorSettings(dna::Structure structure)
    : fogdensity(0)
{
    culler = "crystalspace.culling.frustvis";
    parseSettings(structure);
}

SectorSettings::~SectorSettings()
{
}

void SectorSettings::parseSettings(dna::Structure structure)
{
    // Parse Properties
    dna::ListIterator i
	= structure.getVariable("prop.first").getValueListIterator();
    while(i.hasNext()) {
	dna::Structure propstructure = i.next();
	Property* property = new Property(propstructure);

	if (!parseProperty(property)) {
	    std::cerr << "Warning: Ignoring unknown Property '"
		<< property->getName() << "' in sectorsettings."
		<< std::endl;
	}
    }
}

void SectorSettings::parseBoolProperty(const Property* property, bool& var)
{
    try {
	var = property->getValueBool();
    } catch (dna::Error e) {
	std::cerr << "Couldn't parse property '" << property->getName() << 
	"': Couldn't parse boolean value '" << property->getValue() << "'"
	<< " in worldsettings" << std::endl;
    }
}

bool SectorSettings::parseProperty(const Property* property)
{
    const std::string& pname = property->getName();
    
    if (pname == "fogcolor") {
	try {
	    fogcolor.scanString(property->getValue().c_str());
	} catch (dna::Error& e) {
	    std::cerr << "Couldn't parse property fogcolor in sectorsettings: "
		<< e.what() << std::endl;
	}
	return true;
    }
    if (pname == "fogdensity") {
	try {
	    float newdensity;
	    if (sscanf(property->getValue().c_str(), "%f", &newdensity) != 1)
		throw dna::Error("parse error");
	    fogdensity = newdensity;
	} catch (dna::Error& e) {
	    std::cerr << "Couldn't parse property fogdensity in sectorsettings:"
                << e.what() << std::endl;
	}
	return true;
    }
    if (pname == "culler") {
    	culler = property->getValue();
	return true;
    }
    
    return false;
}

} // end of namespace blend2cs

