#ifndef __BLEND2CS_TERRAINMESH_H__
#define __BLEND2CS_TERRAINMESH_H__

#include <vector>
#include <dna++/structure.h>

#include "vector3d.h"
#include "triangle3d.h"
#include "box2d.h"
#include "polygon.h"
#include "vectoriterator.h"
#include "mesh.h"
#include "meshflags.h"

namespace blend2cs
{

class Object;

class TerrainFace
{
public:
    Triangle3D tri;
    int material_idx;
    // Average color of this triangle from all vertices.
    unsigned char red;
    unsigned char green;
    unsigned char blue;
};

typedef std::vector<TerrainFace> TerrainFaces;

class TerrainMesh : public Mesh
{
public:
    TerrainMesh(const dna::Structure& structure, const MeshFlags& flags);
    ~TerrainMesh();

    /** returns true if the mesh can be written as a factory */
    bool supportFactory() const
    { return true; }

    virtual dna::Structure getStructure() const
    { return structure; }

    const MeshFlags& getFlags() const
    { return flags; }

    virtual void write(XmlWriter& writer) const;
    virtual const char* getMeshPluginName() const
    { return "terrain"; }
    virtual const char* getMeshFactoryName() const
    { return "terrainfact"; }
    const Box2D& getBounds() const { return bounds; }
    float getHeightMin() const { return minheight; }
    float getHeightMax() const { return maxheight; }

    bool saveHeightMap(const std::string& path) const;
    bool saveBaseMaterial(const std::string& path) const;
    bool saveMaterialPalette(const std::string& path) const;

    const std::vector<std::string> getMaterialPalette() const
    { return material_palette; }

private:
    // Find the right material index for this material (for the
    // terrain material palette).
    int findMaterialIdx(const std::string& material);

    // Put the triangle in 'terrainfaces' array everywhere the
    // corresponding point is in the 2D projection of the triangle.
    void distributeTri(const Triangle3D& tri,
	const std::string& material, float dx, float dy,
	int red, int green, int blue);

    // Make a triangle from the DNA structures.
    void makeTri(
	const std::vector<Vector3D>& vertices,
	dna::Structure mface, dna::Structure tface,
	int skip,
	Triangle3D& tri, std::string& material,
	int& red, int& green, int& blue);

    void parseMesh();
    void generateHeightmap();

    dna::Structure structure;

    Box2D bounds;
    Box2D scanbounds;
    float minheight, maxheight;

    int terrain_resolution;
    TerrainFaces* terrainfaces;
    std::vector<std::string> material_palette;

    float* heightmap;
    int* materialmap;
    unsigned char* basematerial;
};

} // end of namespace blend2cs

#endif

