#ifndef __BLEND2CS_THINGMESH_H__
#define __BLEND2CS_THINGMESH_H__

#include <vector>
#include <dna++/structure.h>

#include "vector3d.h"
#include "polygon.h"
#include "vectoriterator.h"
#include "mesh.h"
#include "meshflags.h"

namespace blend2cs
{

class Object;

class ThingMesh : public Mesh
{
public:
    ThingMesh(const dna::Structure& structure, const MeshFlags& flags,
            bool dummy = false, bool eliminateEmptyTriangles = true,
	    bool dontWarnTexture = false);
    ~ThingMesh();

    typedef VectorIterator<Vector3D> VertexIterator;
    VertexIterator getVertexIterator() const;

    const Vector3D& getVertex(int idx) const
    { return vertices[idx]; }
    size_t getVertexCount() const
    { return vertices.size(); }

    typedef VectorIterator<Polygon*> PolygonIterator;
    PolygonIterator getPolygonIterator() const;
    size_t getPolygonCount() const
    { return polygons.size(); }

    /** returns true if the mesh can be written as a factory */
    bool supportFactory() const
    { return supportfactory; }

    virtual dna::Structure getStructure() const
    { return structure; }

    const MeshFlags& getFlags() const
    { return flags; }

    virtual void write(XmlWriter& writer) const;
    virtual const char* getMeshPluginName() const
    { return "thing"; }
    virtual const char* getMeshFactoryName() const
    { return "thingfact"; }

private:
    void parseMesh(bool eliminateEmptyTriangles, bool dontWarnTexture);
    void parseVertices();
    void parseFaces(bool eliminateEmptyTriangles, bool dontWarnTexture);

    bool badUV(dna::Structure tface,dna::Structure mface);
    void makePolys(dna::Structure mface, dna::Structure tface, int skip, bool
            eliminateEmptyTriangles);
    bool emptyPolygon(const Polygon* polygon);

    dna::Structure structure;

    typedef std::vector<Vector3D> Vertices;
    Vertices vertices;
    std::vector<Polygon*> polygons;

    bool supportfactory;
};

} // end of namespace blend2cs

#endif

