#ifndef __BLEND2CS_TRIANGLE3D_H__
#define __BLEND2CS_TRIANGLE3D_H__

#include <string>
#include "vector3d.h"
#include "vector2d.h"
#include "box2d.h"

namespace blend2cs
{

class Triangle3D
{
public:
    Triangle3D() { }
    ~Triangle3D() { }

    Triangle3D(const Triangle3D& o) {
        p1 = o.p1;
        p2 = o.p2;
        p3 = o.p3;
    }

    void set(const Vector3D& newp1, const Vector3D& newp2, const Vector3D& newp3) {
	p1 = newp1;
	p2 = newp2;
	p3 = newp3;
    }

    const Vector3D& getPoint1() const
    { return p1; }
    const Vector3D& getPoint2() const
    { return p2; }
    const Vector3D& getPoint3() const
    { return p3; }

    // Intersect this triangle with a segment between 'v1' and 'v2'.
    bool intersect(const Vector3D& v1, const Vector3D& v2,
    	Vector3D& isect);
    bool intersectVert(const Vector2D& point, float ystart, float ystop, float& yisect);

    // Compute the bounds of this triangle in 2D! This means that
    // the y component will be ignored.
    Box2D getBoundsXZ() const;

    // Check if the given point is in the triangle. This is a 2D test,
    // the y component is ignored.
    bool in(const Vector2D& v) const;

private:
    Vector3D p1, p2, p3;
};

}

#endif

