#ifndef __BLEND2CS_TRIANGLEMESH_H__
#define __BLEND2CS_TRIANGLEMESH_H__

#include <vector>
#include "mesh.h"
#include "vector3d.h"
#include "vectoriterator.h"

namespace blend2cs
{

class ThingMesh;

class TriangleMesh : public Mesh
{
public:
    TriangleMesh(const ThingMesh& mesh);
    ~TriangleMesh();

    virtual dna::Structure getStructure() const
    { return structure; }

    struct Triangle
    {
	Triangle(int v1, int v2, int v3)
	{
	    v[0] = v1; v[1] = v2; v[2] = v3;
	}

	int v[3];
    };
    typedef VectorIterator<Triangle> TriangleIterator;
    TriangleIterator getTriangleIterator() const;
    int getTriangleCount() const
    { return triangles.size(); }
        
    struct Vertex
    {
	Vertex(const Vector3D& newposition, float newu, float newv)
	    : position(newposition), u(newu), v(newv)
	{ }

	Vector3D position;
	float u, v;
    };
    typedef VectorIterator<Vertex> VertexIterator;
    VertexIterator getVertexIterator() const;
    int getVertexCount() const
    { return vertices.size(); }

    const std::string& getMaterial() const
    { return material; }

    virtual void write(XmlWriter& writer) const;

    virtual const char* getMeshFactoryName() const
    { return "genmeshfact"; }
    virtual const char* getMeshPluginName() const
    { return "genmesh"; }

    virtual bool supportFactory() const;

protected:
    /** transforms a normal mesh into the triangulated structure */
    void convertMesh(const ThingMesh& mesh);

    dna::Structure structure;
    
    std::string material;
    std::vector<Vertex> vertices;
    std::vector<Triangle> triangles;

    // used for implementing the conversion algorithm
    friend class MeshConverter;
};

} // end of namespace blend2cs

#endif

