#ifndef __BLEND2CS_VECTOR3D_H__
#define __BLEND2CS_VECTOR3D_H__

#include <string>
#include <valarray>

namespace blend2cs
{

class XmlWriter;

typedef std::valarray<float> V3DArrayType ;

class Vector3D
{
public:
    Vector3D(float x=0, float y=0, float z=0);
    ~Vector3D();

    void set(float newx, float newy, float newz)
    {
	x = newx;
	y = newy;
	z = newz;
    }

    void setX(float newx)
    { x = newx; }
    void setY(float newy)
    { y = newy; }
    void setZ(float newz)
    { z = newz; }

    float getX() const
    { return x; }
    float getY() const
    { return y; }
    float getZ() const
    { return z; }

    void scanString(const char* string);

    void operator += (const Vector3D& other)
    { x += other.x; y += other.y; z += other.z; }
    void operator -= (const Vector3D& other)
    { x -= other.x; y -= other.y; z -= other.z; }

    inline friend Vector3D operator-(const Vector3D& v1, const Vector3D& v2)
    { return Vector3D(v1.x-v2.x, v1.y-v2.y, v1.z-v2.z); }
    inline friend Vector3D operator+(const Vector3D& v1, const Vector3D& v2)
    { return Vector3D(v1.x+v2.x, v1.y+v2.y, v1.z+v2.z); }
    inline friend float operator*(const Vector3D& v1, const Vector3D& v2)
    { return v1.x*v2.x + v1.y*v2.y + v1.z*v2.z; }
    /// Take the cross product of two vectors.
    inline friend Vector3D operator%(const Vector3D& v1, const Vector3D& v2)
    {
      return Vector3D (v1.y*v2.z-v1.z*v2.y,
                       v1.z*v2.x-v1.x*v2.z,
                       v1.x*v2.y-v1.y*v2.x);
    }

    /// Multiply a vector and a scalar.
    inline friend Vector3D operator* (const Vector3D& v, float f)
    { return Vector3D(v.x*f, v.y*f, v.z*f); }
    /// Multiply a vector and a scalar.
    inline friend Vector3D operator* (float f, const Vector3D& v)
    { return Vector3D(v.x*f, v.y*f, v.z*f); }

    void scale(const Vector3D& other)
    { x *= other.x; y *= other.y; z *= other.z; }

    bool operator == (const Vector3D& other) const
    { return x == other.x && y == other.y && z == other.z; }

    bool operator != (const Vector3D& other) const
    { return ! (*this == other); }

    void vaset()
    { fa[0]=x;  fa[1]=y;  fa[2]=z;}
    
    const V3DArrayType& getValArray() 
    { vaset();  return fa; }

    void write(XmlWriter& writer, const char* name) const;

private:
    float x,y,z;
    V3DArrayType fa;
};

}

#endif

