#include <config.h>

#include <iostream>
#include <dna++/listiterator.h>
#include <dna++/variable.h>
#include <dna++/error.h>

#include "worldsettings.h"

namespace blend2cs
{ 

WorldSettings::WorldSettings(dna::Structure structure)
  : clearzbuf(true), clearscreen(true), thingstatic(false),
    genmeshstatic(false), global_renderloop(false)
{
    texturepath = "";
    terrainpath = "";
    renderloop = "standard";
    ambient_set = false;
    parseSettings(structure);
}

WorldSettings::~WorldSettings()
{
}

void WorldSettings::parseSettings(dna::Structure structure)
{
    // Parse Properties
    dna::ListIterator i
	= structure.getVariable("prop.first").getValueListIterator();
    while(i.hasNext()) {
	dna::Structure propstructure = i.next();
	Property* property = new Property(propstructure);

	if (!parseProperty(property)) {
	    std::cerr << "Warning: Ignoring unknown Property '"
		<< property->getName() << "' in worldsettings."
		<< std::endl;
	}
    }
}

void WorldSettings::parseBoolProperty(const Property* property, bool& var)
{
    try {
	var = property->getValueBool();
    } catch (dna::Error e) {
	std::cerr << "Couldn't parse property '" << property->getName() << 
	"': Couldn't parse boolean value '" << property->getValue() << "'"
	<< " in worldsettings" << std::endl;
    }
}

bool WorldSettings::parseProperty(const Property* property)
{
    const std::string& pname = property->getName();

    if (pname == "clearzbuf") {
	parseBoolProperty(property, clearzbuf);
	return true;
    }
    if (pname == "clearscreen") {
	parseBoolProperty(property, clearscreen);
	return true;
    }
    if (pname == "thingstatic") {
	parseBoolProperty(property, thingstatic);
	return true;
    }
    if (pname == "genmeshstatic") {
	parseBoolProperty(property, genmeshstatic);
	return true;
    }

    if (pname == "ambient") {
      try {
	ambient.scanString(property->getValue().c_str());
	ambient_set = true;
      } catch (dna::Error& e) {
	std::cerr << "Couldn't parse property ambient in worldsettings: "
		  << e.what() << std::endl;
      }
      return true;
    }

    if (pname == "texturepath") {
        texturepath = property->getValue();
	return true;
    }

    if (pname == "terrainpath") {
        terrainpath = property->getValue();
	return true;
    }

    if (pname == "renderloop") {
        renderloop = property->getValue();
	return true;
    }
    if (pname == "global_renderloop") {
	parseBoolProperty(property, global_renderloop);
	return true;
    }

    return false;
}

} // end of namespace blend2cs
