#include <config.h>

#include "xmlwriter.h"

#include <dna++/error.h>

namespace blend2cs
{

XmlWriter::XmlWriter(std::ostream& outstream)
    : out(outstream), indent(0)
{
}

XmlWriter::~XmlWriter()
{
    if (sections.begin() != sections.end())
	throw dna::Error("not all xml sections closed.");

    closeTag();
}

void XmlWriter::openSection(const char* name)
{
    newLine();
    out << "<" << name;
    closetag = ">";
    indent++;

    sections.push_back(name);
}

void XmlWriter::closeSection(const char* name)
{
    if (sections.begin() == sections.end())
	throw dna::Error("got closeSection without prior openSection.");

    const std::string& lastsection = * (sections.end() - 1);
    if (lastsection != name)
	throw dna::Error ("mismtach in open/closeSection. Expected %s, got %s.",
		lastsection.c_str(), name);
    sections.pop_back();
    
    indent--;
    newLine();
    // XXX: We should check for consistency here
    out << "</" << name;
    closetag = ">" ;
}

void XmlWriter::writeTag(const char* name)
{
    newLine();
    out << "<" << name;
    closetag = "/>";
    lasttag = name;
}

void XmlWriter::newLine()
{
    closeTag();
    for (int i=0;i<indent;i++)
	out << "  ";
}

void XmlWriter::closeTag()
{
    if (closetag != "")
	out << closetag << "\n";
}

} // end of namespace blend2cs
