#include <typeinfo>
#include <iostream>
#include <fstream>
#include <string>
#include <cmath>
#include <vector>
#include <cstdio>
#include <set>
#include <map>

#include "blendconv.h"
#include "blendobj.h"
#include "mesh.h"
#include "light.h"

// Declare our static variables.
std::string BlenderObject::meshTypeNames[8]=  // Our meshtypes.
  {"","thing","sprite.2D","sprite.3D","genmesh"};
std::set<std::string > BlenderObject::allMySectors;
std::set<std::string > BlenderObject::allMyScenes;

extern std::string sectorManagement;

BlenderObject::BlenderObject()
{
  ID=0;    // Set to invalid ID#
  data=NULL;
  meshType=1;   // Default to thing.
  written=false;  // This object has yet to be written out 
}

BlenderObject::~BlenderObject()
{
}

void BlenderObject::Extract(unsigned int objectID)
{
  Variable temp;
  int i;
  char sb[256];
  int structID;
  char *strval;
  Variable datum;
  Property prop;
  float *boat;    

  TransformNode::Extract(objectID);

  // type of object.
  type=FindVariable(objectID,"type").shortValue[0];  
  
  temp=FindVariable(objectID,"dloc[3]");
  for(i=0;i<3;i++) {
    translation.X[i]+=temp.floatValue[i];
  }

  temp=FindVariable(objectID,"drot[3]");
  for(i=0;i<3;i++) {
    rotation.X[i]+=temp.floatValue[i];
  }

  temp=FindVariable(objectID,"dsize[3]");
  for(i=0;i<3;i++) {
    scale.X[i]+=temp.floatValue[i];
  }

  temp=FindVariable(objectID,"*parent");
  if (temp.pointerValue) parentObjid=FindStructure(temp.pointerValue);
  else parentObjid=0;

  temp=FindVariable(objectID,"lay");  // Read layer information.
  layer=temp.intValue[0];

  // The code below reads & parses Property information 
  // Every property gets turned into a 'key'.  Some keys are
  // Keywords for this program.  Right now keywords can be:
  // 1. "MeshType" (the kind of mesh an object will be in CS) and 
  // 2. "Sector" (The sector in which this object resides).
  temp=FindVariable(objectID,"prop");
  temp=FindVariable(FindStructure(temp.pointerValue),"*first");
  if (temp.pointerValue!=0) {
    std::cout << std::endl;
    std::cout <<"Object Has properties: " << std::endl;
  }
  while  (temp.pointerValue!=0) {    // Loop through all the properties.
    //Find the structure for this property
    structID=FindStructure(temp.pointerValue);   
    temp=FindVariable(structID,"name[32]");  // Name of our property.
    // Print property name.
    std::cout << "      " << temp.stringValue <<"   "<< std::flush;
    prop.propertyName=temp.stringValue;
    temp=FindVariable(structID,"type");  // Type of the property.
    datum=FindVariable(structID,"data");  // Value of most properties
    // is an integer and must be converted.
    prop.enumeration=datum.intValue[0];  
    prop.type=(BlenderProperty) (int) temp.shortValue[0];
    switch (prop.type) {
    case Float:    // This property is a float.
    case Timer:   // Or it is a timer, which is also stored as a float
      boat=((float *) & (prop.enumeration));
      std::cout << "is a float: " << *boat;
      sprintf(sb,"%f",*boat);
      break;
    case String:             // This property is a string.
      datum=FindVariable(structID,"*poin");  // pointer to the string?.
      strval=getString(FindStructure(datum.pointerValue,false));
      if (strval) {
	std::cout << "is a string " ;
	strcpy(sb,strval);
      }
      break;
    case Integer:     // property is an integer.
      std::cout << "is an integer, " << prop.enumeration;
      sprintf(sb,"%d",prop.enumeration);
      break;
    case Huh:       // What the heck is this?
      break;  
    case Boolean:    // property is a boolean
      if (prop.enumeration) sprintf(sb,"true");
      else sprintf(sb, "false");
      std::cout << "is a boolean " << sb ;
      break; 
    }
    prop.stringValue=std::string(sb);
    
    std::cout<< "  " << prop.stringValue << std::endl; 
    // Stuff  that property into a container for later printing.
    propMap.insert(std::make_pair(prop.propertyName,prop));
    
    // Move on to the next property.  They are link-listed together.
      temp=FindVariable(structID,"*next");
  }
  
  PropertyMap::iterator piter;
  Property *pp;  // pointer to properties that we get back.
  // look in the map for keywords.
  if ((piter=propMap.find("MeshType"))!=propMap.end()) {
    pp=&(piter->second);
    switch (pp->type) { 
    case String:  //  Maybe  It's a string
      for (i=0 ; i< 8 ;i++) if (meshTypeNames[i]==pp->stringValue)
	meshType=i;
      break;
    case Integer:   // if it's an integer...
      meshType=pp->enumeration;
      break;
    default:
      break;
    }
    if (meshTypeNames[meshType]!="thing" && 
	meshTypeNames[meshType]!="genmesh") { 
      std::cout << std::endl<< "Warning! unimplemented mesh type " << 
	pp->stringValue << ".  Using mesh type " << meshTypeNames[1] <<
	" instead."<< std::endl;
      meshType=1;
    }
  }

  if ((piter=propMap.find("RenderPriority"))!=propMap.end()) {
    pp=&(piter->second);
    /*   
    // Here we could check if the keyword tells us what render priority
    // to use, if I actually understood the purpose of render priorities.
    switch (piter->type) { 
    case string_bprop:  //  Maybe  It's a string
    rederPriority=(piter->stringValue) ;
    break;
    case integer_bprop:   // if it's an integer...
    break;
    }
    */
  }

  std::string sectName;

  // Here we determine into which sector we should place this object.
  // First, we set up the default
  sectName="room";
  // then check what sort of sector management we're using.
  if (sectorManagement=="Scene") {
    // The user wants objects in different scenes to be in different sectors.
    sectName=*sceneName;   // The sector will be the name of the scene
  } else if (sectorManagement=="Layer") {
    // The user wants objects in different scenes and layers to be in
    // different sectors.  Extend what we did above.
    sprintf(sb,"+Layer.%d",layer);
    sectName = *sceneName+sb;
  }
  // Lastly, if the user has specified a sector via the "Sector" keyword,
  // this overrides everything else.
  if ((piter=propMap.find("Sector"))!=propMap.end()) {  //Look for sectors
    sectName=(piter->second).stringValue;
  }
  // Put the sector name into the sector set and set an iterator to it.
  sectorName=allMySectors.insert(sectName).first;
  

}

Vector BlenderObject::YZSwap(Vector a)
{
  Vector b;
  b.X[0]= a.X[0];
  b.X[1]= a.X[2];
  b.X[2]= a.X[1];
  return b;
}

/*
Vector BlenderObject::CSlocation()
{
  Vector out;
  int j,k;
  // Calculate the location of this object in CS coordinates.
  calcRotationCoeff();
  // apply rotation and translation to 
  for(j=0;j<3;j++) {
    out.X[j]=0.;
    for(k=0;k<3;k++) {
        out.X[j]+=rotCoeff[j].X[k]*.X[k];
    }
  }
}
*/

void BlenderObject::SetMeshFactoryName()
{ // This routine sets the name of the mesh factory that this object might use.
  meshFactoryName=(((Mesh *) data)->name) + std::string(".") +
    meshTypeNames[meshType];
}

void BlenderObject::calcRotationCoeff()
{ // this routine calculates the rotation coefficients for an object;
  // This rotation includes a reflection about the y-z axis, so it is
  // not suitable for direct consumption by CS.

  double rotX, rotY, rotZ; // shortcuts to the rotation vector
  rotX=rotation.X[0];
  rotY=rotation.X[1];
  rotZ=rotation.X[2];
  
  rotCoeff[0].X[0]=cos(rotY)*cos(rotZ);
  rotCoeff[0].X[1]=(sin(rotX)*sin(rotY)*cos(rotZ))-(cos(rotX)*sin(rotZ));
  rotCoeff[0].X[2]=(cos(rotX)*sin(rotY)*cos(rotZ))+(sin(rotX)*sin(rotZ));
  
  rotCoeff[2].X[0]=cos(rotY)*sin(rotZ);
  rotCoeff[2].X[1]=(sin(rotX)*sin(rotY)*sin(rotZ))+(cos(rotX)*cos(rotZ));
  rotCoeff[2].X[2]=(cos(rotX)*sin(rotY)*sin(rotZ))-(sin(rotX)*cos(rotZ));
  
  rotCoeff[1].X[0]=-sin(rotY);
  rotCoeff[1].X[1]=sin(rotX)*cos(rotY);
  rotCoeff[1].X[2]=cos(rotX)*cos(rotY);

}


void BlenderObject::PrintTransform(std::ofstream &output)
{
  // This routine prints out the "<move>" section for each object, giving
  // its relative position, size and orientation with respect to 
  // its particular mesh.  This routine assumes that the vertices
  // of the meshes have been preswapped in y and z.

  unsigned int i;
  int j;
  Vector modRotCoeff[3]; // rotation coefficients

  calcRotationCoeff();    // Calculate rotation coefficients:

  // Now add scaling and a swap to our rotation coefficients so 
  // they can be consumed by CS.  These will be applied to mesh vertex
  // locations (which have already been swapped in y and z) by CS.

  modRotCoeff[0].X[0]=scale.X[0]* rotCoeff[0].X[0];
  modRotCoeff[0].X[2]=scale.X[1]* rotCoeff[0].X[1];
  modRotCoeff[0].X[1]=scale.X[2]* rotCoeff[0].X[2];
  
  // Remember, y and z are swapped between CS and blender.
 
  modRotCoeff[2].X[0]=scale.X[0]*  rotCoeff[2].X[0];
  modRotCoeff[2].X[2]=scale.X[1]*  rotCoeff[2].X[1];
  modRotCoeff[2].X[1]=scale.X[2]*  rotCoeff[2].X[2];
  
  modRotCoeff[1].X[0]=scale.X[0]*  rotCoeff[1].X[0];
  modRotCoeff[1].X[2]=scale.X[1]*  rotCoeff[1].X[1];
  modRotCoeff[1].X[1]=scale.X[2]*  rotCoeff[1].X[2]; 

  if (meshTypeNames[meshType]=="thing") 
    output<< "      <hard";
  else output << "      <";
  output<< "move>"<< std::endl <<"         <matrix>"<< std::endl;

  // Now we print out the rotation matrix values.
  for(i=0;i<3;i++) for(j=0;j<3;j++)
    output << "          <m" << i+1 << j+1 << ">" <<  modRotCoeff[i].X[j] 
           << "</m" << i+1 << j+1 << ">" << std::endl;
  output << "        </matrix>" << std::endl;

  // Print the translation values, swapping y and z again.
  output<< "        <v " << MakeCoordinateString(translation) << " />"<< std::endl ;

  output << "      </";
  if (meshTypeNames[meshType]=="thing") output << "hard";
  output << "move>"<< std::endl;
}

void BlenderObject::propertyPrint(std::ofstream &output)
{
  PropertyMap::iterator piter; 

  for (piter=propMap.begin(); piter!=propMap.end(); piter++) {
    output << "      <key name=\"" << piter->second.propertyName << 
      "\" value=\"" << piter->second.stringValue << "\" />"<< std::endl ;
  } 
}

void BlenderObject::Print(std::ofstream &output)
{
  // Print out properties (keys).

  switch (type) {
  case blender_empty:   // It's an empty.  Make it a node.
    output << "   <node name=\"" << name << "\">"<< std::endl ;
    output << "      <position " << MakeCoordinateString(translation) << "/>"
	   << std::endl;
    propertyPrint(output);  // Print any keys associated with node.
    output << "   </node>"<< std::endl;  
    break;
  case blender_camera:   // This is a camera
    // eh...
    break;
  case blender_mesh:  // Object is a mesh
    // Check to see if this object has a parent that is a mesh.  If it
    // does, don't write it out now.  It will be written out when we recurse.
    if (parentObjid) if (parentObj->second.type==blender_mesh) return;
    // Print this Mesh and all of its children.
    RecurseMeshPrint(output);  
    break;

  case blender_light:  // Object is a light
    if (!data || typeid(*data)!=typeid(Light))  {  
      std::cerr<< "Stupid check failed. No light data or data isn't a light."
	       << std::endl;
      exit(1);
    }
    output << "    <light name=\"" << name << "\">" << std::endl;
    output << "      <center " << MakeCoordinateString(translation) << "/>"
	   << std::endl;
    ((Light *) data)->Write(output,0);
    propertyPrint(output); 
    output << "    </light>" << std::endl;
    break;
  default:
    break;
    
  }
}

void BlenderObject::RecurseMeshPrint(std::ofstream &output)
{
  /// This routine prints out our  mesh objects, and all the mesh
  // Children.

   if (written) { // Another stupid check.
      std::cerr<< "Stupid check failed. This object was already written."
               << std::endl;
      exit(1);
    }
      
    if (!data || typeid(*data)!=typeid(Mesh)) {  
      std::cerr<< "Stupid check failed. No mesh data or data isn't mesh. "
	       << name << "  " << data << std::endl;
      exit(1);
    }
    output << "   <meshobj name=\"" << name << "\">"<< std::endl;
    output << "      <zuse/>" << std::endl;
    output << "      <plugin>crystalspace.mesh.loader." 
           << meshTypeNames[meshType] << "</plugin>"<< std::endl;
    // If our mesh has portals, we don't get it's info from a factory.
    // We print it out ourselves.
    if ((((Mesh *) data)->HasPortals())) { 
      // Copy our property information over first.
      ((Mesh *) data)->PropertyMapCopy(propMap);
      ((Mesh *) data)->Write(output,meshType);
    } else {
      output << "      <params>"<< std::endl;
      //      output << "        <factory>" <<((Mesh *)data)->name <<"</factory>"<< std::endl;
      output << "        <factory>" << meshFactoryName <<"</factory>"<< std::endl;
      output << "      </params>"<< std::endl;
    }
    PrintTransform(output); // Print the object's rotation, location,etc.
    output << "      <priority>level</priority>" << std::endl;
    propertyPrint(output);
    
    std::vector< BObjectContainer::iterator >::iterator chiter;

    // Ok, we've printed this mesh, now print all of it's children.
    for (chiter=childset.begin(); chiter!=childset.end(); chiter++) {
      BlenderObject *bo= &((*chiter)->second);  // The child object.
      // Put in a quick dummy check to make certain that our parent-child
      // matching went well.
      if (bo->parentObjid != ID) { 
        std::cerr<< "Poo. I screwed the pooch on this.  object's child's "
          "parent is not the object." << std::endl;
        exit(1);
      }  
      if (bo->type==blender_mesh) { // Only print child now if it is a mesh.
	// Set the child's coordinates relative to the parents.
	bo->translation.X[0]+=translation.X[0];
	bo->translation.X[1]+=translation.X[1];
	bo->translation.X[2]+=translation.X[2];
	bo->RecurseMeshPrint(output);  // Print the child's data.
	
      }
    }

    output << "   </meshobj>"<< std::endl; 
    // Mark This object as printed.  
    written=true;
}

