#ifndef BLENDOBJ_H
#define BLENDOBJ_H

#include <fstream>
#include <string>
#include <vector>
#include <set>
#include <map>
#include "blendconv.h"
#include "transnode.h"

enum BlenderObjType { blender_empty, blender_mesh, blender_2,blender_3,
		      blender_4,blender_5,blender_6,blender_7,blender_8,
		      blender_9, blender_light, blender_camera};

struct StartInfo {
  Vector position;
  Vector up;
  Vector forward;
};


class BlenderObject;

typedef std::map<int, BlenderObject> BObjectContainer;

class BlenderObject : public TransformNode {
 public:
  static std::string meshTypeNames[8];
  static std::set<std::string > allMySectors;
  static std::set<std::string > allMyScenes;

  BlenderObject();
  
  virtual ~BlenderObject(); // {};
 
  void Extract(unsigned int objectID);

  virtual void Write(std::ofstream &output, int type=0) {};
  void Print(std::ofstream &output);
  void propertyPrint(std::ofstream &output);
  void calcRotationCoeff();
  Vector YZSwap(Vector);
  void SetMeshFactoryName();

  Node *data;

  short type;  // type of object.  Follows Blender's enumeration.
  //  int ID;
  int meshType;  // Type of mesh in CS-- thing, genmesh, spriteXD...
  std::set<std::string >::iterator sectorName;  //"pointer" to sectors name
  // "pointer" to Scene in which object resides.
  std::set<std::string >::iterator sceneName; 

  int parentObjid;   // parent's pointer value.
  BObjectContainer::iterator parentObj;   //pointer to parent.
  // make a set of iterators that point to the child objects.
  std::vector< BObjectContainer::iterator > childset;

  bool written;
  // Name of mesh factory that this object might use.
  std::string meshFactoryName; 
  
 protected:
  void PrintTransform(std::ofstream &output);
  void RecurseMeshPrint(std::ofstream &output);


  Vector rotCoeff[3]; // Rotation coefficients.

  PropertyMap propMap;  //Map of the properties.
  int layer;   // Layer in which object resides.
  std::string renderPriority; // Not implemented yet.

};


#endif
