#include <iostream>
#include <iomanip>
#include <cmath>
#include <set>
#include <typeinfo>
#include "level.h"
#include <map>

void Level::BuildLevel(std::set<Mesh, NodeCmp<Mesh> > *meshes, 
		       std::set<Light,NodeCmp<Light> > *lights,
		       BObjectContainer *instances) 

{
  this->lights=lights;
  this->meshes=meshes;
  this->instances=instances;
}

void Level::Write(const char *filename, Textures *textures)
{
  using namespace std;
  std::ofstream output(filename);

  if(!output) {
    std::cerr << "fatal: unable to open output file " << filename << std::endl;
    exit(-1);
  }

  output << setprecision(10);

  output << "<world>" << std::endl;

  textures->Write(output);

  output << "  <plugins>" << std::endl;
  output << "    <plugin name=\"thing\">crystalspace.mesh.loader.thing</plugin>" << std::endl;
  output << "  </plugins>" << std::endl;

  output << "  <renderpriorities>" << std::endl;
  output << "    <priority name=\"level\">" << std::endl;
  output << "       <level>1</level>" << std::endl;
  output << "       <sort>NONE</sort>" << std::endl;
  output << "    </priority>" << std::endl;
  output << "  </renderpriorities>" << std::endl;

  output << "  <settings>" << std::endl;
  output << "    <clearscreen>yes</clearscreen>" << std::endl;
  output << "    <clearzbuf>yes</clearzbuf>" << std::endl;
  output << "  </settings>" << std::endl;

  BObjectContainer::iterator oiter;
  BlenderObject *bo;
  // Iterate through objects to find a camera and use it's start 
  // location.  We should probably put this code elsewhere.
  for (oiter=instances->begin(); oiter!=instances->end(); oiter++) {
    bo= & (oiter->second);  
    if (bo->type==blender_camera) {   // Camera
      output << "  <start>"<<std::endl<< "    <sector>"<< 
	*(bo->sectorName) << "</sector>"<<std::endl<<"    <position " << 
	bo->MakeCoordinateString(bo->translation) << " />"<<std::endl<<
	"  </start>"<< std::endl;

      break;  // Just use the first camera that we find.
    }
  }


  // We write out mesh factories.  We do this by iterating over the
  // objects, because some objects may have the same meshes, but want
  // different mesh types, i.e. the same mesh data could be a sprite3D
  // or a thing.  As a consequence we keep track of which mesh has
  // been printed out in which format.

  // As an exception however, meshes with portals can have unique
  // sector destinations so we do not write them out as factories,
  // We write them out as unique meshes below.  This makes the
  // code much messier

  Mesh *meshi;

  for (oiter=instances->begin(); oiter!=instances->end(); oiter++) {
    bo= & (oiter->second); 
    meshi=(Mesh *) bo->data;
    if (meshi) {
      if (bo->type == blender_mesh ) {  // Is the object a mesh?
      	if (typeid(*meshi)==typeid(Mesh)) {  // Is the data a mesh?
	  if (!meshi->HasPortals()) {
	    meshi->WriteFactory(output, bo->meshFactoryName,bo->meshType);
	  }
	}
      }
    }
  }

  /*
  for(unsigned int i=0;i<meshes.size();i++) {
    meshes[i].MeshObjHeader(output);
    meshes[i].Write(output);
  }
  */

  // Here we iterate over the sector names. Only do objects if they're in the
  // right sector.  Inefficient, but simple.
  for (std::set<std::string >::iterator seciter=
	 BlenderObject::allMySectors.begin();
       seciter!=BlenderObject::allMySectors.end(); seciter++)  {
    
    output << "  <sector name=\"" << *seciter << "\">" << std::endl;
    
    // Now we write out all the objects
    for (oiter=instances->begin(); oiter!=instances->end(); oiter++) {
      bo= & (oiter->second); 
      if (*(bo->sectorName)==*seciter)//If this object is in our sector
	bo->Print(output);  // Print it out.
    }
        
    /*
      for(unsigned int j=0;j<lights->size();j++) {
      (*lights)[j].Write(output);
      }
    */
    output << "  </sector>" << std::endl;
  }

  output << "</world>" << std::endl;   // The end.
}
