#ifndef MESH_H
#define MESH_H

#include <fstream>
#include <vector>
#include <string>
#include <map>
#include <utility>

#include "transnode.h"
#include "texture.h"
#include "polygon.h"

struct Vertex {
  Vector coords;  // This is 3 double values.
  short Normals[3];
};

class Mesh : public TransformNode {
 public:
  Mesh();

  void Extract(int meshID, Textures *textures);
  void Write(std::ofstream &output,int type=0);
  void WriteFactory(std::ofstream &output, std::string &factName,int type=0);
  bool HasPortals();
  void PropertyMapCopy(PropertyMap &pMap);
  //  int getUVcoords(int vertexNumber);
  void GetUVCoords(int vertexNumber, double **uv);
  void WriteVertices(std::ofstream &output);
  void WriteTriangles(std::ofstream &output);
  
  
 protected:
  void ApplyTransformation();
  void ReduceData();
  
  bool typesWritten[8];
  bool hasAnyPortals;  // Boolean indicating whether this mesh has portals.
  bool hasUVCoords;
  //  std::vector<Vector> Vertices;
  std::vector<Vertex> Vertices;
  std::vector<Poly> Polys;
  
  std::string textureFile;
  std::string textureName;
  
  Textures *textures;
};



#endif
