#ifndef NODE_H
#define NODE_H

#include "property.h"

/**
 * This Node is a base class for all blender objects. It keeps
 * track of properties and tranformations that were extracted
 * from the matching DNA structure.
 */
class Node {
 public:
  /// The type of the Node
  enum NodeType {
    TNode,
    TEmpty,
    TMesh,
    TLight,
    TCamera
  };
  
  Node() { nodeType=TNode; };

  virtual void Extract(unsigned int ID);

  /// Return the type of the Node
  NodeType GetType() { return nodeType; };
  /// Return the structure ID of the node
  unsigned int GetID() { return ID; };

  virtual void Write(std::ofstream &output, int type=0) {};
  virtual void WriteFactory(std::ofstream &output, std::string &factName,int type=0) {};

  std::string name;
  unsigned int ID;
  
 protected:
  NodeType nodeType;

  PropertyMap propMap;  //Map of the properties.
};

/** 
 * This is an operator object for comparing Nodes by ID
 */
template <class C> struct NodeCmp {
  bool operator()(C x,C  y) const
  {
    return x.GetID() < y.GetID();
  }
};

#endif
