#include <fstream>

#include "polygon.h"
#include "blendobj.h"
#include "texture.h"


void Poly::WriteThing(std::ofstream &output,PropertyMap &pm, Textures *tx)
{
  if (textureID<0) return; //Don't output a face if there is no texture.

  /*  This routine writes out a polygon for a CS thing mesh. */
  WriteThingSide(output,pm,tx,0); // Write out at least one side

  // Write out other side if the polygon is "double sided"
  if (faceMode.ms.double_sided)   WriteThingSide(output,pm,tx,1); 
}

void Poly::WriteThingSide(std::ofstream &output, PropertyMap &properties,  
			  Textures *tx,int numSide)
{ /* This routine actually writes out the polygon */
  // if nside=0, we write out the usual polygon side.
  // Otherwise we write out the opposite side of the polygon.

  std::string NameOfCurrentTexture;
  int j;

  output << "      <p>" << std::endl;  // Start of polygon def.
  
  // Only things have portals and multiple textures
  // Texture for this face.
  
  NameOfCurrentTexture=tx->GetName(textureID);
  
  // If this face is a portal, look in the properties for a match to
  // the texturename.  If we match, use property's value as a
  // sectorname.  The property names should have been copied to this
  // mesh object by the referring object.
  if (isPortal) {
    PropertyMap::iterator piter; 
    if ((piter=properties.find(NameOfCurrentTexture))!=properties.end()) { 
      output << "        <portal><sector>"<< piter->second.stringValue 
	     << "</sector></portal>"<< std::endl;
    }
  } else 
    // Output a face specific texture
    output << "        <material>" << NameOfCurrentTexture
	   << "</material>" << std::endl;

  // write out if this face has no collision detection enabled.
  if (!faceMode.ms.collide) 
    output <<   "        <colldet>no</colldet>" << std::endl;
  
  if (numSide==0) {  // Doing first side of the polygon 
    // write in inversed order to get normals inversed correctly in CS
    for(j=2;j>=0;j--) { 
      output << "        <v>"<< Vertices[j] << "</v>"<< std::endl;
    }
    output << "        <texmap>" << std::endl;
    for(j=2;j>=0;j--) {
      output << "          <uv idx=\"" << 2-j << "\"" << " u=\"" << UV[j][0] <<
	"\" v=\"" << -UV[j][1] << "\"/>" << std::endl; 
    }
  } else { //Write out opposite side of polygon if poly is double sided
    for(j=0;j<3;j++) { 
      output << "        <v>"<< Vertices[j] << "</v>"<< std::endl;
    }
    output << "        <texmap>" << std::endl;
    for(j=0;j<3;j++) {
      output << "          <uv idx=\"" << j << "\"" << " u=\"" << UV[j][0] << 
	"\" v=\"" << -UV[j][1] << "\"/>" << std::endl; 
    }
  }
  
  output << "        </texmap>" << std::endl;
  output << "      </p>" << std::endl;
}

