#include <iostream>
#include <iomanip>
#include <cmath>

#include "sprite.h"
#include "mesh.h"
#include "blendconv.h"

void Sprite::Extract(int objectIndex, int meshID, Textures *textures)
{
  Variable *temp;
  int i, j;
  int firstVertex;
  int numVertices;
  int firstFace;
  int numFaces;
  bool hasUVCoords;
  int firstUVFace;
  int materialID;
  int imageID;

  // Read object basics:
  ExtractObject(objectIndex);

  // Read the name of the mesh:
  temp=FindVariable(meshID,"id");
  temp=FindVariable(FindStructure(temp->pointerValue),"name[24]");
  meshName=temp->stringValue;
  
  // Extract the number of vertices and faces:
  temp=FindVariable(meshID,"totvert");
  numVertices=temp->intValue[0];
  temp=FindVariable(meshID,"totface");
  numFaces=temp->intValue[0];
  
  // Extract the IDs of the first vertex and face:
  // XXX: The following code assumes that MVert, TFace and MFace data structures are alligned as arrays!
  temp=FindVariable(meshID,"*mvert");
  firstVertex=FindStructure(temp->pointerValue);
  temp=FindVariable(meshID,"*mface");
  firstFace=FindStructure(temp->pointerValue);
  temp=FindVariable(meshID,"*tface");
  if(temp->pointerValue==0) {
    hasUVCoords=false;
  } else {
    firstUVFace=FindStructure(temp->pointerValue);
    hasUVCoords=true;
  }
  
  // Extract the vertex coodinates:
  Vertices.resize(numVertices);
  for(i=0;i<numVertices;i++) {
    temp=FindVariable(firstVertex+i,"co[3]");
    for(j=0;j<3;j++) {
      Vertices[i].X[j]=temp->floatValue[j];
    }
  }

  // Extract faces and UV coordinates:
  for(i=0;i<numFaces;i++) {
    temp=FindVariable(firstFace+i,"v4");
    if(temp->shortValue[0]==0) { // XXX: Is there a better way to determine that?
      // This face is a triangle
      Polys.resize(Polys.size()+1);
      temp=FindVariable(firstFace+i,"v1");
      Polys[Polys.size()-1].Vertices[0]=temp->shortValue[0];
      temp=FindVariable(firstFace+i,"v2");
      Polys[Polys.size()-1].Vertices[1]=temp->shortValue[0];
      temp=FindVariable(firstFace+i,"v3");
      Polys[Polys.size()-1].Vertices[2]=temp->shortValue[0];
      
      if(hasUVCoords) {
	temp=FindVariable(firstUVFace+i,"uv[4][2]");
	for(j=0;j<3;j++) {
	  Polys[Polys.size()-1].UV[j*2][0]=temp->floatValue[j*2];
	  Polys[Polys.size()-1].UV[j*2][1]=temp->floatValue[j*2+1];
	}
      } else {
	// TODO: invent some UV coordinates here
      }
    } else {
      // This face is a quad -> split it and store it in two polys
      Polys.resize(Polys.size()+2);
      temp=FindVariable(firstFace+i,"v1");
      Polys[Polys.size()-2].Vertices[0]=temp->shortValue[0];
      temp=FindVariable(firstFace+i,"v2");
      Polys[Polys.size()-2].Vertices[1]=temp->shortValue[0];
      temp=FindVariable(firstFace+i,"v3");
      Polys[Polys.size()-2].Vertices[2]=temp->shortValue[0];
      temp=FindVariable(firstFace+i,"v3");
      Polys[Polys.size()-1].Vertices[0]=temp->shortValue[0];
      temp=FindVariable(firstFace+i,"v4");
      Polys[Polys.size()-1].Vertices[1]=temp->shortValue[0];
      temp=FindVariable(firstFace+i,"v1");
      Polys[Polys.size()-1].Vertices[2]=temp->shortValue[0];
      
      if(hasUVCoords) {
	temp=FindVariable(firstUVFace+i,"uv[4][2]");
	Polys[Polys.size()-2].UV[0][0]=temp->floatValue[0];
	Polys[Polys.size()-2].UV[0][1]=temp->floatValue[1];
	Polys[Polys.size()-2].UV[1][0]=temp->floatValue[2];
	Polys[Polys.size()-2].UV[1][1]=temp->floatValue[3];
	Polys[Polys.size()-2].UV[2][0]=temp->floatValue[4];
	Polys[Polys.size()-2].UV[2][1]=temp->floatValue[5];
	
	Polys[Polys.size()-1].UV[0][0]=temp->floatValue[4];
	Polys[Polys.size()-1].UV[0][1]=temp->floatValue[5];
	Polys[Polys.size()-1].UV[1][0]=temp->floatValue[6];
	Polys[Polys.size()-1].UV[1][1]=temp->floatValue[7];
	Polys[Polys.size()-1].UV[2][0]=temp->floatValue[0];
	Polys[Polys.size()-1].UV[2][1]=temp->floatValue[1];
      } else {
	// TODO: invent some UV coordinates here
      }
    }
  }

  std::cout << "extracted " << Vertices.size() << " vertices and " << Polys.size() << " polygons" << std::endl;

  ApplyTransformation();
  ReduceData();

  std::cout << "reduced to " << Vertices.size() << " vertices and " << Polys.size() << " polygons" << std::endl;

  // extract texture file name:
  // retrieve pointer to Material structure:
  temp=FindVariable(meshID,"**mat");
  materialID=FindStructure(temp->pointerValue);
  // retrieve pointer to MTex structure (not much useful data there):
  temp=FindVariable(materialID,"*mtex[8]");
  // retrieve pointer to Tex structure (not much useful data there, either):
  temp=FindVariable(FindStructure(temp->pointerValue),"*tex");
  // retrieve pointer to Image structure:
  temp=FindVariable(FindStructure(temp->pointerValue),"*ima");
  imageID=FindStructure(temp->pointerValue);
  // retrieve image file name from structure:
  temp=FindVariable(imageID,"name[160]");
  textureFile=temp->stringValue;
  std::cout << "texture file name is: " << textureFile << std::endl;

  textureName=textures->StripPath(textureFile);
  this->textures=textures;
  textures->Add(textureFile);
}

void Sprite::Write(std::ofstream &output)
{
  int i, j;

  output << "    <meshobj name=\"" << objectName << "\">" << std::endl;
  output << "      <plugin>thing</plugin>" << std::endl;
  output << "      <params>" << std::endl;

  for(i=0;i<Vertices.size();i++) {
    output << "      <v x=\"" << Vertices[i].X[0] << "\" y=\"" << Vertices[i].X[1] << "\" z=\"" << Vertices[i].X[2] << "\"" << "/>" << std::endl;
  }

  output << "      <material>" << textureName << "</material>" << std::endl;

  for(i=0;i<Polys.size();i++) {
    output << "      <p>" << std::endl;
    for(j=0;j<3;j++) {
      output << "        <v>" << Polys[i].Vertices[j] << "</v>" << std::endl;
    }
    output << "        <texmap>" << std::endl;
    for(j=0;j<3;j++) {
      output << "          <uv idx=\"" << j << "\"" << " u=\"" << Polys[i].UV[j][0] << "\" v=\"" << Polys[i].UV[j][1] << "\"/>" << std::endl; 
    }
    output << "        </texmap>" << std::endl;
    output << "      </p>" << std::endl;
  }

  output << "      </params>" << std::endl;
  output << "      <zuse/>" << std::endl;
  output << "      <priority>level</priority>" << std::endl;
  output << "    </meshobj>" << std::endl;
}

void Sprite::ApplyTransformation()
{
  int i, j, k;
  double rotX, rotY, rotZ; // shortcuts to the rotation vector
  Vector rotCoeff[3]; // rotation coefficients
  Vector temp;

  rotX=rotation.X[0];
  rotY=rotation.X[1];
  rotZ=rotation.X[2];
  
  // Precalculate rotation coefficients:
  rotCoeff[0].X[0]=cos(rotY)*cos(rotZ);
  rotCoeff[0].X[1]=(sin(rotX)*sin(rotY)*cos(rotZ))-(cos(rotX)*sin(rotZ));
  rotCoeff[0].X[2]=(cos(rotX)*sin(rotY)*cos(rotZ))+(sin(rotX)*sin(rotZ));
  
  rotCoeff[1].X[0]=cos(rotY)*sin(rotZ);
  rotCoeff[1].X[1]=(sin(rotX)*sin(rotY)*sin(rotZ))+(cos(rotX)*cos(rotZ));
  rotCoeff[1].X[2]=(cos(rotX)*sin(rotY)*sin(rotZ))-(sin(rotX)*cos(rotZ));
  
  rotCoeff[2].X[0]=-sin(rotY);
  rotCoeff[2].X[1]=sin(rotX)*cos(rotY);
  rotCoeff[2].X[2]=cos(rotX)*cos(rotY);
  
  for(i=0;i<Vertices.size();i++) {
    for(j=0;j<3;j++) {
      Vertices[i].X[j]*=scale.X[j];
      temp.X[j]=0;
    }
    
    for(j=0;j<3;j++) {
      for(k=0;k<3;k++) {
	temp.X[j]+=rotCoeff[j].X[k]*Vertices[i].X[k];
      }
    }

    for(j=0;j<3;j++) {
      Vertices[i].X[j]=temp.X[j]+translation.X[j];
    }

    // we need to rotate the scene by -90 degrees along z axis (engine coord system requirement):
    temp.X[0]=Vertices[i].X[1];
    Vertices[i].X[1]=Vertices[i].X[2];
    Vertices[i].X[2]=temp.X[0];

    // HACK: work around a bug in CS that crashes the renderer when z==0
    if(Vertices[i].X[2]==0.0) {
      std::cerr << "WARNING: got a vertex with final Z coordinate being exactly 0 -- adding small offset" << std::endl;
      Vertices[i].X[2]+=0.000001;
    }
  }

  // HACK: we need to adjust the UV coordinates delivered by blender
  for(i=0;i<Polys.size();i++) {
    for(j=0;j<3;j++) {
      Polys[i].UV[j][1]*=-1;
    }
  }
}

void Sprite::ReduceData()
{
  int i, j, k, l;
  int numMatches;
  bool isEqual, isValid;
  std::vector<int> vertexMap;
  std::vector<Vector> newVertices;
  std::vector<Poly> newPolys;

  // try to eliminate duplicate vertices:
  
  vertexMap.resize(Vertices.size());

  for(i=0;i<Vertices.size();i++) {
    isEqual=false;
    for(j=0;j<newVertices.size();j++) {
      if((float)Vertices[i].X[0]==(float)newVertices[j].X[0] &&
	 (float)Vertices[i].X[1]==(float)newVertices[j].X[1] &&
	 (float)Vertices[i].X[2]==(float)newVertices[j].X[2]) {      
	isEqual=true;
	vertexMap[i]=j;
	break;
      }
    }
    if(!isEqual) {
      newVertices.push_back(Vertices[i]);
      vertexMap[i]=newVertices.size()-1;
    }
  }

  Vertices=newVertices;

  for(i=0;i<Polys.size();i++) {
    for(j=0;j<3;j++) {
      Polys[i].Vertices[j]=vertexMap[Polys[i].Vertices[j]];
    }
  }

  // try to eliminate invalid/duplicate polygons:
  for(i=0;i<Polys.size();i++) {
    if(Polys[i].Vertices[0]==Polys[i].Vertices[1] ||
       Polys[i].Vertices[0]==Polys[i].Vertices[2] ||
       Polys[i].Vertices[1]==Polys[i].Vertices[2]) {
      std::cerr << "WARNING: found polygon with only two distinct vertices -- dropping" << std::endl;
      isValid=false;
    } else {
      isValid=true;
    }
    if(isValid) {
      isEqual=false;
      for(j=0;j<newPolys.size();j++) {
	numMatches=0;
	for(k=0;k<3;k++) {
	  for(l=0;l<3;l++) {
	    if(Polys[i].Vertices[k]==Polys[j].Vertices[l]) {
	      numMatches++;
	    }
	  }
	}
	if(numMatches==3) {
	  isEqual=true;
	  break;
	} else if(numMatches>3) {
	  std::cerr << "INTERNAL ERROR: numMatches > 3" << std::endl;
	  exit(-1);
	}
      }
      if(!isEqual) {
	newPolys.push_back(Polys[i]);
      }
    }
  }
  Polys=newPolys;
}
