// $Id: ExternalExec.cpp 72 2003-10-24 23:58:51Z felfert $

#include "stdafx.h"
#include "ExternalExec.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#include <string>
#include <process.h>
#include "debug.h"

#undef DIRECTIPC

typedef enum handleFlags {
	HF_STDIN_R	=  1,
	HF_STDIN_W	=  2,
	HF_STDOUT_R =  4,
	HF_STDOUT_W =  8,
	HF_STDERR_R = 16,
	HF_STDERR_W = 32,
} ThandleFlags;

class Process
{
public:
	bool CreateStdPipe(TpipeTypes, HANDLE);
	bool Startup(long *, Process *nextProcess = NULL);
	void SetWorkingDir(const char *);
	void SetParams(const char *);
	void SetExecutable(const char *);
	void SetName(const char *);
	void SetIpcBuffer(int);
	void SetShowWindow(int);
	void SetCriticalSections(CRITICAL_SECTION*, CRITICAL_SECTION*);
	DWORD LastError();

	HANDLE StdHandle(TpipeTypes);
	PROCESS_INFORMATION *ProcessInfo();
	const char * WorkingDir(bool nullOk = false);
	const char * Params(bool nullOk = false);
	const char * Executable(bool nullOk = false);
	const char * Name(bool nullOk = false);
	DWORD ExitCode();
	int ShowWindow();
	void Terminate();

	Process(const char *executable, const char *params = NULL, const char *wdir = NULL, int showWindow = SW_SHOW);
	virtual ~Process();

private:
	std::string			m_strExecutable;
	std::string			m_strParameters;
	std::string			m_strWorkingDir;
	std::string			m_strName;
	int					m_iShowWindow;
	int					m_iIpcBuffer;
	bool				m_bRunning;
	DWORD				m_dwExitCode;
	DWORD				m_dwLastError;
	HANDLE				m_hControlThread;
	HANDLE				m_hPumpThread;
	HANDLE				m_hPumpTo;
	HANDLE				m_hStdHandle[6];
	bool				m_bPrivate[6];
	PROCESS_INFORMATION	m_sProcessInfo;
	CRITICAL_SECTION	*m_sHandleModify;
	CRITICAL_SECTION	*m_sProcessCountAccess;
	Process				*m_pcNextProcess;
	long				*m_piParentProcessCount;

	void closeAllHandles();
	void killProcess(bool);
	void ipcPump();

	static void controller(LPVOID);
	static void ipcPumpStub(LPVOID);
};

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ExternalExec::ExternalExec()
	: m_pfCallBack(NULL)
	, m_pfWaitCallBack(NULL)
	, m_hStderrThread(NULL)
	, m_hStdoutThread(NULL)
	, m_hStdinThread(NULL)
	, m_bCloseStdin(true)
{
	InitializeCriticalSection(&m_sHandleModify);
	InitializeCriticalSection(&m_sProcessCountAccess);
	m_hStdinWriteEvent = CreateEvent(NULL, FALSE, FALSE, "ExternalExec::m_hStdinWriteEvent");
	m_hStdinWrittenEvent = CreateEvent(NULL, FALSE, FALSE, "ExternalExec::m_hStdinWrittenEvent");
	for (int i = 0; i < 6; i++)
		m_hStdHandle[i] = INVALID_HANDLE_VALUE;
	m_piProcessCount = (long *)GlobalAlloc(GPTR, sizeof(long));
	*m_piProcessCount = 0;
	m_strStdinBuf = "";
}

ExternalExec::~ExternalExec()
{
	ExitCode();
	Clear();
	CloseHandle(m_hStdinWrittenEvent);
	CloseHandle(m_hStdinWriteEvent);
	DeleteCriticalSection(&m_sHandleModify);
	DeleteCriticalSection(&m_sProcessCountAccess);
	GlobalFree(m_piProcessCount);
}

DWORD
ExternalExec::ExitCode(bool flush /* = true */)
{
	DWORD res = STILL_ACTIVE;
	EnterCriticalSection(&m_sProcessCountAccess);
	while (*m_piProcessCount) {
		LeaveCriticalSection(&m_sProcessCountAccess);
		if (m_pfWaitCallBack)
			m_pfWaitCallBack(m_lpvCallbackParam);
		else
			Sleep(100);
		EnterCriticalSection(&m_sProcessCountAccess);
	}
	LeaveCriticalSection(&m_sProcessCountAccess);
	if (m_vProcessList.size())
		res = m_vProcessList[m_vProcessList.size()-1]->ExitCode();
	while (flush && ((m_hStdHandle[2] != INVALID_HANDLE_VALUE) ||
		(m_hStdHandle[2] != INVALID_HANDLE_VALUE))) {
		if (m_pfWaitCallBack)
			m_pfWaitCallBack(m_lpvCallbackParam);
		else
			Sleep(100);
	}
	return res;
}

bool
ExternalExec::Running()
{
	bool ret;
	EnterCriticalSection(&m_sProcessCountAccess);
	ret = (*m_piProcessCount != 0);
	LeaveCriticalSection(&m_sProcessCountAccess);
	return ret;
}

void
ExternalExec::Kill()
{
	cleanup();
}

bool
ExternalExec::Clear()
{
	if (Running())
		return false;

	cleanup();
	for (int i = 0; i < m_vProcessList.size(); i++)
		delete m_vProcessList[i];
	m_vProcessList.clear();
	m_strStdinBuf = "";
	m_pfCallBack = NULL;
	m_pfWaitCallBack = NULL;
	m_hStderrThread = NULL;
	m_hStdoutThread = NULL;
	m_hStdinThread = NULL;
	m_bCloseStdin = true;
	return true;
}

void
ExternalExec::SetCallBack(runCallBack cbFunc)
{
	m_pfCallBack = cbFunc;
}

void
ExternalExec::SetWaitCallBack(waitCallBack cbFunc)
{
	m_pfWaitCallBack = cbFunc;
}

void
ExternalExec::SetIpcBuffer(int bsize)
{
	m_iIpcBuffer = bsize;
	for (int i = 0; i < m_vProcessList.size(); i++)
		m_vProcessList[i]->SetIpcBuffer(bsize);
}

void
ExternalExec::cleanup()
{
	bool handleValid;
	int i;

	for (i = 0; i < m_vProcessList.size(); i++)
		m_vProcessList[i]->Terminate();

	EnterCriticalSection(&m_sHandleModify);
	handleValid = (m_hStdHandle[4] != INVALID_HANDLE_VALUE);
	LeaveCriticalSection(&m_sHandleModify);
	if (handleValid)
		WaitForSingleObject(m_hStderrThread, INFINITE);

	EnterCriticalSection(&m_sHandleModify);
	handleValid = (m_hStdHandle[2] != INVALID_HANDLE_VALUE);
	LeaveCriticalSection(&m_sHandleModify);
	if (handleValid)
		WaitForSingleObject(m_hStdoutThread, INFINITE);

	EnterCriticalSection(&m_sHandleModify);
	handleValid = (m_hStdHandle[1] != INVALID_HANDLE_VALUE);
	LeaveCriticalSection(&m_sHandleModify);
	if (handleValid) {
		m_bCloseStdin = true;
		SetEvent(m_hStdinWriteEvent);
		WaitForSingleObject(m_hStdinThread, INFINITE);
	}

	EnterCriticalSection(&m_sHandleModify);
	for (i = 0; i < 6; i++) {
		if (m_hStdHandle[i] != INVALID_HANDLE_VALUE)
			CloseHandle(m_hStdHandle[i]);
		m_hStdHandle[i] = INVALID_HANDLE_VALUE;
	}
	LeaveCriticalSection(&m_sHandleModify);
}

bool
ExternalExec::createStdPipe(TpipeTypes which)
{
	SECURITY_ATTRIBUTES saAttr;
	BOOL res = FALSE;

	// Set the bInheritHandle flag so pipe handles are inherited. 
	saAttr.nLength = sizeof(SECURITY_ATTRIBUTES); 
	saAttr.bInheritHandle = TRUE; 
	saAttr.lpSecurityDescriptor = NULL; 


	int i = which << 1;
	res = CreatePipe(&m_hStdHandle[i], &m_hStdHandle[i+1], &saAttr, 0);
	if (res) {
		HANDLE hTemp;
		switch (which) {
			case PT_STDIN:
				// Make our side of the pipe noninheritable. 
				res = DuplicateHandle(GetCurrentProcess(), m_hStdHandle[i+1],
					GetCurrentProcess(), &hTemp , 0, FALSE, DUPLICATE_SAME_ACCESS);
				if (res) {
					CloseHandle(m_hStdHandle[i+1]);
					m_hStdHandle[i+1] = hTemp;
				} else
					m_dwLastError = DUPLICATEHANDLE_FAILED;
				break;
			case PT_STDOUT:
			case PT_STDERR:
				res = DuplicateHandle(GetCurrentProcess(), m_hStdHandle[i],
					GetCurrentProcess(), &hTemp , 0, FALSE, DUPLICATE_SAME_ACCESS);
				if (res) {
					CloseHandle(m_hStdHandle[i]);
					m_hStdHandle[i] = hTemp;
				} else
					m_dwLastError = DUPLICATEHANDLE_FAILED;
				break;
		}
		if (!res) {
			CloseHandle(m_hStdHandle[i]);
			CloseHandle(m_hStdHandle[i+1]);
		}
	} else
		m_dwLastError = CREATEPIPE_FAILED;
	if (!res)
		m_hStdHandle[i] = m_hStdHandle[i+1] = INVALID_HANDLE_VALUE;
	return (res == TRUE);
}

void
ExternalExec::stdinPump(LPVOID lpv)
{
	ExternalExec *p = (ExternalExec *)lpv;
	_ASSERT(p);
	dbg("ExternalExec: stdinPump starting");
	p->inPump();
	dbg("ExternalExec: stdinPump terminating");
	EnterCriticalSection(&p->m_sHandleModify);
	p->m_hStdHandle[1] = INVALID_HANDLE_VALUE;
	LeaveCriticalSection(&p->m_sHandleModify);
	dbg("ExternalExec: stdinPump finished");
}

void
ExternalExec::stdoutPump(LPVOID lpv)
{
	ExternalExec *p = (ExternalExec *)lpv;
	_ASSERT(p);
	dbg("ExternalExec: stdoutPump starting");
	p->outPump(PT_STDOUT);
	dbg("ExternalExec: stdoutPump terminating");
	EnterCriticalSection(&p->m_sHandleModify);
	p->m_hStdHandle[2] = INVALID_HANDLE_VALUE;
	LeaveCriticalSection(&p->m_sHandleModify);
	dbg("ExternalExec: stderrPump finished");
}


void
ExternalExec::stderrPump(LPVOID lpv)
{
	ExternalExec *p = (ExternalExec *)lpv;
	_ASSERT(p);
	dbg("ExternalExec: stderrPump starting");
	p->outPump(PT_STDERR);
	dbg("ExternalExec: stderrPump terminating");
	EnterCriticalSection(&p->m_sHandleModify);
	p->m_hStdHandle[4] = INVALID_HANDLE_VALUE;
	LeaveCriticalSection(&p->m_sHandleModify);
	dbg("ExternalExec: stderrPump finished");
}

void
ExternalExec::inPump()
{
	DWORD dwCount;

	while (!m_bCloseStdin) {
		WaitForSingleObject(m_hStdinWriteEvent, INFINITE);
		if (!m_bCloseStdin) {
			if ((!WriteFile(m_hStdHandle[1], m_strStdinBuf.c_str(), m_strStdinBuf.size(), &dwCount, NULL))
				|| (dwCount != m_strStdinBuf.size())) {
				CloseHandle(m_hStdHandle[1]);
				m_bCloseStdin = true;
			}
		}
		m_strStdinBuf = "";
		PulseEvent(m_hStdinWrittenEvent);
	}

	WriteFile(m_hStdHandle[1], NULL, 0, &dwCount, NULL);
	Sleep(100);
	CloseHandle(m_hStdHandle[1]);
	m_bCloseStdin = true;
}

void
ExternalExec::outPump(TpipeTypes channel)
{
	char ch;
	BOOL cbId;
	DWORD dwCount;
	HANDLE h;
	bool doPump = true;
	bool doFlush = false;
	std::string linebuf = "";

	switch (channel) {
		case PT_STDOUT:
			h = m_hStdHandle[2];
			cbId = FALSE;
			break;
		case PT_STDERR:
			h = m_hStdHandle[4];
			cbId = TRUE;
			break;
		default:
			return; // Something went extremely wrong!
	}

	while (doPump) {
		if ((!ReadFile(h, &ch, 1, &dwCount, NULL)) || (dwCount == 0)) {
			doPump = false;
			doFlush = true;
		} else {
			switch (ch) {
				case '\n':
					doFlush = true;
					// fall thru
				default:
					linebuf += ch;
					break;
			}
		}
		if (doFlush) {
			if (!linebuf.empty()) {
				if (m_pfCallBack) {
					if (!m_pfCallBack(m_lpvCallbackParam, cbId, linebuf.c_str())) {
						EnterCriticalSection(&m_sHandleModify);
						for (int i = 0; i < 6; i++) {
							if (m_hStdHandle[i] != INVALID_HANDLE_VALUE)
								CloseHandle(m_hStdHandle[i]);
							m_hStdHandle[i] = INVALID_HANDLE_VALUE;
						}
						LeaveCriticalSection(&m_sHandleModify);
					}
				}
				linebuf = "";
			}
			doFlush = false;
		}
	}
	CloseHandle(h);
}

bool
ExternalExec::Run(LPVOID cbParam/* = NULL */, bool needStdin /* = false */)
{
	int pcount = 0;

	m_lpvCallbackParam = cbParam;

	m_hStdinThread = NULL;
	m_hStdoutThread = NULL;
	m_hStderrThread = NULL;
	m_dwLastError = 0;
	if (!m_vProcessList.size()) {
		m_dwLastError = PROCESSLIST_EMPTY;
		return false;
	}

	if (!createStdPipe(PT_STDIN)) {
		err("ExternalExec: Creating main stdin pipe FAILED");
		m_dwLastError = PROCESSLIST_EMPTY;
		return false;
	}

	if (!createStdPipe(PT_STDOUT)) {
		err("ExternalExec: Creating main stdout pipe FAILED");
		cleanup();
		return false;
	}
	if (!createStdPipe(PT_STDERR)) {
		err("ExternalExec: Creating main stderr pipe FAILED");
		cleanup();
		return false;
	}
	dbg("Main pipes created, handles:\n  Parent: I=%08x O=%08x E=%08x\n  Child:  I=%08x O=%08x E=%08x",
			m_hStdHandle[1], m_hStdHandle[2], m_hStdHandle[4], m_hStdHandle[0], m_hStdHandle[3], m_hStdHandle[5]);

	// Create all necessary child pipes
	for (int i = 0; i < m_vProcessList.size(); i++) {
		Process *p = m_vProcessList[i];

		// first process in chain gets its input handle
		// connected to us.
		if (!p->CreateStdPipe(PT_STDIN,
			(i == (m_vProcessList.size() - 1)) ? m_hStdHandle[0] : INVALID_HANDLE_VALUE)) {
			m_dwLastError = p->LastError();
			cleanup();
			err("ExternalExec: Creating STDIN pipes of %s FAILED", p->Name());
			return false;
		}

		// last process in chain gets it stdout handle
		// connected to us. All other processess are
		// connected to the stdin handle of their predecessor.
		if (!p->CreateStdPipe(PT_STDOUT, (i == 0) ? m_hStdHandle[3] : INVALID_HANDLE_VALUE)) {
			m_dwLastError = p->LastError();
			err("ExternalExec: Creating STDOUT pipes of %s FAILED", p->Name());
			cleanup();
			return false;
		}

		// stderr always goes to us.
		if (!p->CreateStdPipe(PT_STDERR, m_hStdHandle[5])) {
			m_dwLastError = p->LastError();
			err("ExternalExec: Creating STDERR pipes of %s FAILED", p->Name());
			cleanup();
			return false;
		}

		// Create process
		if (!p->Startup(m_piProcessCount, i ? m_vProcessList[i-1] : NULL)) {
			m_dwLastError = p->LastError();
			err("ExternalExec: Startup of %s FAILED", p->Name());
			cleanup();
			return false;
		}

		pcount++;
	}

	m_bCloseStdin = true;
	if (pcount) {
		m_bCloseStdin = !needStdin;

		// Close child ends of our main handles. No more needed
		EnterCriticalSection(&m_sHandleModify);
		CloseHandle(m_hStdHandle[0]);
		CloseHandle(m_hStdHandle[3]);
		CloseHandle(m_hStdHandle[5]);
		m_hStdHandle[0] = m_hStdHandle[3] = m_hStdHandle[5] = INVALID_HANDLE_VALUE;
		LeaveCriticalSection(&m_sHandleModify);

		// Start up threads
		m_hStdoutThread = (HANDLE)_beginthread(stdoutPump, 0, this);
		if (!m_hStdoutThread) {
			err("main stdout thread FAILED");
			m_dwLastError = CREATETHREAD_FAILED;
			cleanup();
			return false;
		}
		m_hStderrThread = (HANDLE)_beginthread(stderrPump, 0, this);
		if (!m_hStderrThread) {
			err("main stderr thread FAILED");
			m_dwLastError = CREATETHREAD_FAILED;
			cleanup();
			return false;
		}
		if (needStdin) {
			m_hStdinThread = (HANDLE)_beginthread(stdinPump, 0, this);
			if (!m_hStderrThread) {
				err("main stdin thread FAILED");
				m_dwLastError = CREATETHREAD_FAILED;
				cleanup();
				return false;
			}
		} else {
			EnterCriticalSection(&m_sHandleModify);
			CloseHandle(m_hStdHandle[1]);
			m_hStdHandle[1] = INVALID_HANDLE_VALUE;
			LeaveCriticalSection(&m_sHandleModify);
		}
	}
	return true;
}

bool
ExternalExec::AddProcess(const char *executable, const char *params /* = NULL */,
						 const char *wdir /* = NULL */, int showWindow /* = SW_SHOW */)
{
	// push a new process to front so processes get in reverse order
	Process *p = new Process(executable, params, wdir, showWindow);
	p->SetCriticalSections(&m_sHandleModify, &m_sProcessCountAccess);
	m_vProcessList.push_front(p);

	return true;
}

bool
ExternalExec::Write(const char *buf)
{
	bool handleValid;

	if (!buf)
		return false;
	EnterCriticalSection(&m_sHandleModify);
	handleValid = (m_hStdHandle[1] != INVALID_HANDLE_VALUE);
	LeaveCriticalSection(&m_sHandleModify);

	if (handleValid) {
		m_strStdinBuf += buf;
		SetEvent(m_hStdinWriteEvent);
		WaitForSingleObject(m_hStdinWrittenEvent, INFINITE);
	} else
		m_dwLastError = INVALID_STDINHANDLE;
	return handleValid;
}

bool
ExternalExec::Close()
{
	bool handleValid;

	EnterCriticalSection(&m_sHandleModify);
	m_bCloseStdin = true;
	handleValid = (m_hStdHandle[1] != INVALID_HANDLE_VALUE);
	LeaveCriticalSection(&m_sHandleModify);
	if (handleValid) {
		SetEvent(m_hStdinWriteEvent);
		WaitForSingleObject(m_hStdinThread, INFINITE);
	} else
		m_dwLastError = INVALID_STDINHANDLE;
	return handleValid;
}

// class Process implementation

DWORD
ExternalExec::LastError()
{
	return m_dwLastError;
}

Process::Process(const char *executable, const char *params /* = NULL */,
									   const char *wdir /* = NULL */, int showWindow /* = SW_SHOW */)
									   : m_iShowWindow(showWindow)
									   , m_bRunning(false)
									   , m_hControlThread(INVALID_HANDLE_VALUE)
									   , m_hPumpThread(INVALID_HANDLE_VALUE)
									   , m_hPumpTo(INVALID_HANDLE_VALUE)
									   , m_pcNextProcess(NULL)
									   , m_dwExitCode(STILL_ACTIVE)
									   , m_iIpcBuffer(0)

{
	m_strParameters = params ? params : "";
	m_strWorkingDir = wdir ? wdir : "";
	m_strName = "UNNAMED";
	SetExecutable(executable);
	for (int i = 0; i < 6; i++) {
		m_hStdHandle[i] = INVALID_HANDLE_VALUE;
		m_bPrivate[i] = false;
	}
	m_sProcessInfo.hProcess = INVALID_HANDLE_VALUE;
	m_sProcessInfo.hThread = INVALID_HANDLE_VALUE;
}

Process::~Process()
{
	dbg("~Process(%s)", Name());
	Terminate();
}

void
Process::Terminate()
{
	// Close all handles and give it 2 more
	// seconds to terminate itself
	closeAllHandles();
	WaitForSingleObject(m_hControlThread, 500);
	if (m_bRunning) {
		killProcess(false);
		WaitForSingleObject(m_hControlThread, INFINITE);
	}	
}

void
Process::SetShowWindow(int sw)
{
	if (m_bRunning)
		return;
	m_iShowWindow = sw;
}

int
Process::ShowWindow()
{
	return m_iShowWindow;
}

void
Process::SetIpcBuffer(int bsize)
{
	if (m_bRunning)
		return;
	m_iIpcBuffer = bsize;
}
	
void
Process::SetExecutable(const char *s)
{
	_ASSERT(s);
	if (m_bRunning)
		return;

	m_strExecutable = s;

	m_strName = m_strExecutable;
	m_strName.erase(0, m_strName.rfind('\\') + 1);
	int ext = m_strName.rfind(".exe");
	if (ext != -1)
		m_strName.erase(ext);
	ext = m_strName.rfind(".com");
	if (ext != -1)
		m_strName.erase(ext);
}

void
Process::SetParams(const char *s)
{
	if (m_bRunning)
		return;
	m_strParameters = s ? s : "";
}

void
Process::SetWorkingDir(const char *s)
{
	if (m_bRunning)
		return;
	m_strWorkingDir = s ? s : "";
}

void
Process::SetName(const char *s)
{
	if (m_bRunning)
		return;
	m_strName = s ? s : "";
}

void
Process::SetCriticalSections(LPCRITICAL_SECTION sHandleModify, LPCRITICAL_SECTION sProcessCountAccess)
{
	m_sHandleModify = sHandleModify;
	m_sProcessCountAccess = sProcessCountAccess;
}

const char *
Process::Name(bool nullOk /* = false */)
{
	return (nullOk && m_strName.empty())
		? NULL : m_strName.c_str();
}

const char *
Process::Executable(bool nullOk)
{
	return (nullOk && m_strExecutable.empty())
		? NULL : m_strExecutable.c_str();
}

const char *
Process::Params(bool nullOk)
{
	return (nullOk && m_strParameters.empty())
		? NULL : m_strParameters.c_str();
}

const char *
Process::WorkingDir(bool nullOk)
{
	return (nullOk && m_strWorkingDir.empty())
		? NULL : m_strWorkingDir.c_str();
}

PROCESS_INFORMATION *
Process::ProcessInfo()
{
	if (!m_bRunning)
		return NULL;
	return &m_sProcessInfo;
}

DWORD
Process::ExitCode()
{
	return m_dwExitCode;
}

DWORD
Process::LastError()
{
	return m_dwLastError;
}

bool
Process::CreateStdPipe(TpipeTypes which, HANDLE hParentHandle)
{
	SECURITY_ATTRIBUTES saAttr;
	BOOL res = FALSE;

	if (m_bRunning)
		return false;
	// Set the bInheritHandle flag so pipe handles are inherited. 
	saAttr.nLength = sizeof(SECURITY_ATTRIBUTES); 
	saAttr.bInheritHandle = TRUE; 
	saAttr.lpSecurityDescriptor = NULL; 


	int i = which << 1;
	m_bPrivate[i] = m_bPrivate[i+1] = false;
	if (hParentHandle == INVALID_HANDLE_VALUE) {
		res = CreatePipe(&m_hStdHandle[i], &m_hStdHandle[i+1], &saAttr, 0);
		m_bPrivate[i] = m_bPrivate[i+1] = true;
		if (res) {
			HANDLE hTemp;
			switch (which) {
				case PT_STDIN:
#ifndef DIRECTIPC
					// Make our side of the pipe noninheritable. 
					res = DuplicateHandle(GetCurrentProcess(), m_hStdHandle[i+1],
						GetCurrentProcess(), &hTemp , 0, FALSE, DUPLICATE_SAME_ACCESS);
					if (res) {
						CloseHandle(m_hStdHandle[i+1]);
						m_hStdHandle[i+1] = hTemp;
					} else
						m_dwLastError = DUPLICATEHANDLE_FAILED;
#endif
					break;
				case PT_STDOUT:
#ifdef DIRECTIPC
					CloseHandle(m_hStdHandle[i]);
					CloseHandle(m_hStdHandle[i+1]);
					m_hStdHandle[i] = m_hStdHandle[i+1] = INVALID_HANDLE_VALUE;
					break;
#endif
				case PT_STDERR:
					// Make our side of the pipe noninheritable. 
					res = DuplicateHandle(GetCurrentProcess(), m_hStdHandle[i],
						GetCurrentProcess(), &hTemp , 0, FALSE, DUPLICATE_SAME_ACCESS);
					if (res) {
						CloseHandle(m_hStdHandle[i]);
						m_hStdHandle[i] = hTemp;
					} else
						m_dwLastError = DUPLICATEHANDLE_FAILED;
					break;
			}
			if (!res) {
				CloseHandle(m_hStdHandle[i]);
				CloseHandle(m_hStdHandle[i+1]);
			}
		} else
			m_dwLastError = CREATEPIPE_FAILED;
	} else {
		res = TRUE;
		switch (which) {
			case PT_STDIN:
				m_hStdHandle[0] = hParentHandle;
				break;
			case PT_STDOUT:
				m_hStdHandle[3] = hParentHandle;
				break;
			case PT_STDERR:
				m_hStdHandle[5] = hParentHandle;
				break;
		}
	}
	return (res == TRUE);
}

HANDLE
Process::StdHandle(TpipeTypes which)
{
	switch (which) {
		case PT_STDIN:
			return m_hStdHandle[1];
		case PT_STDOUT:
			return m_hStdHandle[2];
		case PT_STDERR:
			return m_hStdHandle[4];
	}
	return NULL;
}

void
Process::closeAllHandles()
{
	if (m_bRunning)
		return;
	EnterCriticalSection(m_sHandleModify);
	for (int i = 0; i < 6; i++) {
		if ((m_hStdHandle[i] != INVALID_HANDLE_VALUE) && m_bPrivate[i])
			CloseHandle(m_hStdHandle[i]);
		m_hStdHandle[i] = INVALID_HANDLE_VALUE;
		m_bPrivate[i] = false;
	}
	if (m_hPumpTo != INVALID_HANDLE_VALUE) {
		CloseHandle(m_hPumpTo);
		m_hPumpTo = INVALID_HANDLE_VALUE;
	}
	if (m_sProcessInfo.hProcess != INVALID_HANDLE_VALUE) {
		CloseHandle(m_sProcessInfo.hProcess);
		m_sProcessInfo.hProcess = INVALID_HANDLE_VALUE;
	}
	if (m_sProcessInfo.hThread != INVALID_HANDLE_VALUE) {
		CloseHandle(m_sProcessInfo.hThread);
		m_sProcessInfo.hThread = INVALID_HANDLE_VALUE;
	}
	LeaveCriticalSection(m_sHandleModify);
}

bool
Process::Startup(long *parentProcessCount, Process *nextProcess /* = NULL */)
{
	std::string sParams;
	STARTUPINFO siStartInfo;
	BOOL res;

	m_dwLastError = 0;
	m_pcNextProcess = nextProcess;
	m_piParentProcessCount = parentProcessCount;

	ZeroMemory(&m_sProcessInfo, sizeof(PROCESS_INFORMATION));
	m_sProcessInfo.hProcess = INVALID_HANDLE_VALUE;
	m_sProcessInfo.hThread = INVALID_HANDLE_VALUE;

	ZeroMemory( &siStartInfo, sizeof(STARTUPINFO));
	siStartInfo.cb = sizeof(STARTUPINFO);
	siStartInfo.dwFlags = STARTF_USESHOWWINDOW|STARTF_USESTDHANDLES;
	siStartInfo.wShowWindow = m_iShowWindow;
	siStartInfo.hStdInput	= m_hStdHandle[0];
#ifdef DIRECTIPC
	if (m_pcNextProcess) {
		m_hStdHandle[3] = m_pcNextProcess->StdHandle(PT_STDIN);
		m_bPrivate[3] = true;
	}
#endif
	siStartInfo.hStdOutput	= m_hStdHandle[3];
	siStartInfo.hStdError	= m_hStdHandle[5];

	sParams = m_strName;
	if (!m_strParameters.empty()) {
		sParams += " ";
		sParams += m_strParameters;
	}
	
	char exepath[MAX_PATH];
	char exeargs[MAX_PATH];
	lstrcpy(exepath, m_strExecutable.c_str());
	lstrcpy(exeargs, sParams.c_str());

	dbg("Process::Startup(%s): About to create process: '%s'", Name(), exeargs);
	// Create the child process.
	res = CreateProcess(
			exepath,			// path to executable
			exeargs,			// command line 
			NULL,				// process security attributes 
			NULL,				// primary thread security attributes 
			TRUE,				// handles are inherited 
			CREATE_NEW_PROCESS_GROUP,					// creation flags
			// Using CREATE_NEW_PROCESS_GROUP, because we want to be able to send Ctrl-C and Ctrl-Break
			NULL,				// use parent's environment
			WorkingDir(true),	// working dir
			&siStartInfo,		// STARTUPINFO pointer
			&m_sProcessInfo		// receives PROCESS_INFORMATION
		);
	if (res) {
		dbg("Process %s created, handles:\n  Parent: I=%08x%c O=%08x%c E=%08x%c\n  Child:  I=%08x%c O=%08x%c E=%08x%c", Name(),
			m_hStdHandle[1], m_bPrivate[1] ? 'p' : 'g', m_hStdHandle[2], m_bPrivate[2] ? 'p' : 'g',
			m_hStdHandle[4], m_bPrivate[4] ? 'p' : 'g', m_hStdHandle[0], m_bPrivate[0] ? 'p' : 'g',
			m_hStdHandle[3], m_bPrivate[4] ? 'p' : 'g', m_hStdHandle[5], m_bPrivate[5] ? 'p' : 'g');

		m_dwExitCode = STILL_ACTIVE;
		// Start controller thread
		m_hControlThread = (HANDLE)_beginthread(controller, 0, this);
		if (!m_hControlThread) {
			err("controller thread %s FAILED", Name());
			killProcess(true);
			m_dwLastError = CREATETHREAD_FAILED;
			return false;
		}
		while ((!m_bRunning) && (m_dwExitCode == STILL_ACTIVE))
			Sleep(10);
		dbg("controller thread %s started", Name());

		// Close all child-side Handles. These are no more needed for us.
		EnterCriticalSection(m_sHandleModify);
		if (m_bPrivate[0] && (m_hStdHandle[0] != INVALID_HANDLE_VALUE)) {
			CloseHandle(m_hStdHandle[0]);
			m_hStdHandle[0] = INVALID_HANDLE_VALUE;
		}
		if (m_bPrivate[3] && (m_hStdHandle[3] != INVALID_HANDLE_VALUE)) {
			CloseHandle(m_hStdHandle[3]);
			m_hStdHandle[3] = INVALID_HANDLE_VALUE;
		}
		if (m_bPrivate[5] && (m_hStdHandle[5] != INVALID_HANDLE_VALUE)) {
			CloseHandle(m_hStdHandle[5]);
			m_hStdHandle[5] = INVALID_HANDLE_VALUE;
		}
#ifdef DIRECTIPC
		if (m_pcNextProcess && (m_pcNextProcess->StdHandle(PT_STDIN) != INVALID_HANDLE_VALUE))
			CloseHandle(m_pcNextProcess->StdHandle(PT_STDIN));
#endif
		LeaveCriticalSection(m_sHandleModify);
		dbg("child handles of %s closed", Name());
#ifndef DIRECTIPC
		// If a chained process is given, create an additional pump tread
		// from this process (stdout) to the chained process stdin.
		dbg("Check for %s IPC pump", Name());
		EnterCriticalSection(m_sHandleModify);
		if (m_pcNextProcess && (m_pcNextProcess->StdHandle(PT_STDIN) != INVALID_HANDLE_VALUE))
			m_hPumpTo = m_pcNextProcess->StdHandle(PT_STDIN);
		LeaveCriticalSection(m_sHandleModify);
		if (m_hPumpTo != INVALID_HANDLE_VALUE) {
			m_hPumpThread = (HANDLE)_beginthread(ipcPumpStub, 0, this);
			if (!m_hPumpThread) {
				err("IPC pump thread %s FAILED", Name());
				killProcess(false);
				m_dwLastError = CREATETHREAD_FAILED;
				return false;
			}
			dbg("IPC pump thread for %s %08x -> %08x created", Name(), m_hStdHandle[2], m_hPumpTo);
		} else
			dbg("No IPC pump for %s needed", Name());
#endif
	} else {
		m_dwLastError = CREATEPROCESS_FAILED;
		err("Process creation of %s FAILED", Name());
	}
	return (res == TRUE);
}

void
Process::killProcess(bool force)
{
	DWORD res;

	// First, try Ctrl-C, then Ctrl-Break, finally terminate the process
	EnterCriticalSection(m_sHandleModify);
	if (m_bRunning || force) {
		dbg("Trying Ctrl-C on %s", Name());
		GenerateConsoleCtrlEvent(CTRL_C_EVENT, m_sProcessInfo.dwProcessId);
		res = WaitForSingleObject(m_sProcessInfo.hProcess, 500);
		if (res != WAIT_OBJECT_0) {
			dbg("Trying CtrlBreak on %s", Name());
			GenerateConsoleCtrlEvent(CTRL_BREAK_EVENT, m_sProcessInfo.dwProcessId);
			res = WaitForSingleObject(m_sProcessInfo.hProcess, 500);
			if (res != WAIT_OBJECT_0) {
				err("KILLING %s", Name());
				TerminateProcess(m_sProcessInfo.hProcess, (UINT)-1);
			}
		}
	}
	LeaveCriticalSection(m_sHandleModify);
}

void
Process::ipcPumpStub(LPVOID lpv)
{
	Process *p = (Process *)lpv;
	_ASSERT(p);

	dbg("ipcPump %s starting", p->Name());
	p->ipcPump();
	dbg("ipcPump %s finished", p->Name());
}

void
Process::ipcPump()
{
	char ch;
	bool doPump = true;
	bool doFlush = false;
	bool res;
	DWORD dwCount;
	std::string linebuf = "";

	while (doPump) {
		//dbgc("r");
		res = (ReadFile(m_hStdHandle[2], &ch, 1, &dwCount, NULL) == TRUE);
		//dbgc("R");
		if ((!res) || (dwCount != 1)) {
			DWORD dwErr = GetLastError();
			if (dwErr != ERROR_BROKEN_PIPE)
				err("ipcPump(%s): ReadFile failed: dwCount=%d, res=%d Err=%d"
					, Name(), dwCount, res, dwErr);
			doPump = false;
			doFlush = true;
		} else {
			if (m_iIpcBuffer != 0) {
				linebuf += ch;
				if (linebuf.size() >= m_iIpcBuffer)
					doFlush = true;
			} else {
				switch (ch) {
					case '\n':
						doFlush = true;
						// fall thru
					default:
						linebuf += ch;
						break;
				}
			}
		}
		if (doFlush) {
			//dbgc("w");
			res = (WriteFile(m_hPumpTo, linebuf.c_str(), linebuf.size(), &dwCount, NULL) == TRUE);
			//dbgc("W");
			if ((!res) || (dwCount != linebuf.size())) {
				err("ipcPump(%s): WriteFile failed: req=%d dwCount=%d, res=%d Err=%d",
					Name(), linebuf.size(), dwCount, res, GetLastError());
				doPump = false;
			}
			linebuf = "";
			doFlush = false;
		}
	}
	EnterCriticalSection(m_sHandleModify);
	CloseHandle(m_hPumpTo);
	m_hPumpTo = INVALID_HANDLE_VALUE;
	if (m_hStdHandle[2] != INVALID_HANDLE_VALUE) {
		CloseHandle(m_hStdHandle[2]);
		m_hStdHandle[2] = INVALID_HANDLE_VALUE;
	}
	LeaveCriticalSection(m_sHandleModify);
}

void
Process::controller(LPVOID lpv)
{
	Process *p = (Process *)lpv;
	DWORD exitCode = ~STILL_ACTIVE;
	HANDLE h[2];
	_ASSERT(p);

	dbg("controller(%s) starting", p->Name());
	EnterCriticalSection(p->m_sProcessCountAccess);
	*(p->m_piParentProcessCount) += 1;
	LeaveCriticalSection(p->m_sProcessCountAccess);
	p->m_bRunning = true;
	h[0] = p->m_sProcessInfo.hThread;
	h[1] = p->m_sProcessInfo.hProcess;
	do {
		if (p->m_dwExitCode == STILL_ACTIVE)
			dbg("Wait on %s returned but exitCode is STILL_ACTIVE", p->Name());
		WaitForMultipleObjects(2, h, FALSE, INFINITE);
		GetExitCodeProcess(p->m_sProcessInfo.hProcess, &p->m_dwExitCode);
	} while (exitCode == STILL_ACTIVE);

#ifdef DIRECTIPC
	dbg("controller(%s) terminating", p->Name());
#else
	dbg("controller(%s) terminating, waiting for IPC pump to finish", p->Name());
	while (p->m_hPumpTo != INVALID_HANDLE_VALUE)
		Sleep(100);
#endif

	p->m_bRunning = false;
	dbg("controller(%s) finished", p->Name());
	EnterCriticalSection(p->m_sProcessCountAccess);
	*(p->m_piParentProcessCount) -= 1;
	LeaveCriticalSection(p->m_sProcessCountAccess);
}
