// $Id: LogDlg.cpp 72 2003-10-24 23:58:51Z felfert $
//
// LogDlg.cpp : implementation file
//

#include "stdafx.h"
#include "blender2cs.h"
#include "LogDlg.h"
#include "aboutdlg.h"
#include "debug.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// LogDlg dialog

LogDlg::LogDlg(CWnd* pParent /*=NULL*/)
	: CDialog(LogDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(LogDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	m_bRunning = FALSE;
	if (!pParent) {
		AfxMessageBox(IDS_ERR_INTERNAL | MB_ICONSTOP);
		exit(-1);
	}
	settings = (CBlender2csDlg*)pParent;
}


void LogDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(LogDlg)
	DDX_Control(pDX, IDC_STATIC_STATUS, m_cStatus);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(LogDlg, CDialog)
	//{{AFX_MSG_MAP(LogDlg)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_CLOSE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static BOOL myOutCallback(LPVOID ptr, BOOL isErr, const char *line)
{
	((LogDlg *)ptr)->OutCallback(isErr, line);
	return TRUE;
}

static UINT threadProc(LPVOID ptr)
{
	((LogDlg *)ptr)->RunBlend2Cs();
	return 0;
}

/////////////////////////////////////////////////////////////////////////////
// LogDlg message handlers

BOOL LogDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);
	// Dito for IDM_EXIT
	ASSERT((IDM_EXIT & 0xFFF0) == IDM_EXIT);
	ASSERT(IDM_EXIT < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strMenu;
		bool addSeparator = true;

		strMenu.LoadString(IDS_ABOUTBOX);
		if (!strMenu.IsEmpty())
		{
			if (addSeparator) {
				pSysMenu->AppendMenu(MF_SEPARATOR);
				addSeparator = false;
			}
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strMenu);
		}
		strMenu.LoadString(IDS_EDIT_EXIT);
		if (!strMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_EXIT, strMenu);
		}
	}

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	m_cEdit.SubclassWnd(IDC_RICHEDIT_MSG, this);

	AfxBeginThread(threadProc, this);

	return TRUE;  // return TRUE  unless you set the focus to a control
}

void LogDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	switch (nID & 0xFFF0)
	{
		case IDM_ABOUTBOX:
			{
				CAboutDlg dlgAbout;
				dlgAbout.DoModal();
			}
			break;
		case IDM_EXIT:
			OnCancel();
			break;
		default:
			CDialog::OnSysCommand(nID, lParam);
	}
}

void LogDlg::OnClose() 
{
	if (m_bRunning) {
		if (AfxMessageBox(IDS_WARN_RUNNING,
			MB_YESNO | MB_ICONEXCLAMATION | MB_DEFBUTTON2) != IDYES)
			return;
		m_bRunning = FALSE;
		ee.Kill();
	}
	CDialog::OnClose();
}

void LogDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR LogDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void LogDlg::OutCallback(BOOL isErr, CString line)
{
	if (!m_bRunning)
		return;

	m_nLineCount++;
	if ((isErr != m_bLastErr) || ((m_nLineCount % 100) == 0)) {
		m_cEdit.SetColor(m_bLastErr ? 0x000000FF : 0x00FF0000);
		m_cEdit.ReplaceSel(m_sOutBuffer);
		m_bLastErr = isErr;
		m_sOutBuffer = "";
	}
	m_sOutBuffer += line;
	CString tmp;
	tmp.Format(_T("Lines: %d"), m_nLineCount);
	SetDlgItemText(IDC_STATIC_STATUS, tmp);
}

void LogDlg::RunBlend2Cs()
{
	CString args = settings->m_nOutputMode ? _T("-o ") : _T("-l ");
	if (settings->m_bListFile)
		args += _T("-L ");
	if (settings->m_bFixLamps)
		args += _T("-f ");
	if (!settings->m_sTextureDir.IsEmpty()) {
		args += _T("-t \"");
		args += settings->m_sTextureDir + _T("\"");
	}
	args += _T(" \"") + settings->m_sInputFilename + _T("\"");
	args += _T(" \"") + settings->m_sOutputFilename + _T("\"");
	
	// Get blend2cs path from main app

	CString tmp = ((CBlender2csApp*)AfxGetApp())->m_sBlend2CsExecutable;
	ee.AddProcess(tmp, args, NULL, SW_HIDE);
	ee.SetCallBack(myOutCallback);
	m_nLineCount = 0;
	m_bLastErr = FALSE;
	m_sOutBuffer = "";

	if (ee.Run(this)) {
		m_bRunning = TRUE;
		// Disable edit control while we are filling it with text
		m_cEdit.EnableWindow(FALSE);
		AfxGetApp()->DoWaitCursor(1);
		m_cEdit.Clear();
		m_cEdit.SetColor(0x00008000);
		m_cEdit.ReplaceSel(_T("Running blend2cs ") + args + _T(" ...\n"));
		// Next statement blocks until blend2cs is finished
		tmp.Format(_T("blend2cs returned with exit code %d\n"), ee.ExitCode());
		m_nLineCount--;
		OutCallback(!m_bLastErr, "");
		m_cEdit.Clear();
		m_cEdit.SetColor(0x00008000);
		m_cEdit.ReplaceSel(tmp);
		m_cEdit.LineScroll(m_nLineCount);
		// Enable edit control again
		m_cEdit.EnableWindow(TRUE);
		AfxGetApp()->RestoreWaitCursor();
		m_bRunning = FALSE;
	} else {
		int id = IDS_ERR_UNKOWN;
		switch(ee.LastError()) {
		case DUPLICATEHANDLE_FAILED:
			id = IDS_ERR_DUPHANDLE;
			break;
		case CREATEPIPE_FAILED:
			id = IDS_ERR_PIPE;
			break;
		case PROCESSLIST_EMPTY:
			id = IDS_ERR_NOPROC;
			break;
		case CREATETHREAD_FAILED:
			id = IDS_ERR_THREAD;
			break;
		case CREATEPROCESS_FAILED:
			id = IDS_ERR_PROCESS;
			break;
		case INVALID_STDINHANDLE:
			id = IDS_ERR_STDIN;
			break;
		}
		AfxMessageBox(id, MB_ICONSTOP);
		EndDialog(0);
	}
}

void LogDlg::OnCancel() 
{
	if (m_bRunning) {
		if (AfxMessageBox(IDS_WARN_RUNNING,
			MB_YESNO | MB_ICONEXCLAMATION | MB_DEFBUTTON2) != IDYES)
			return;
		m_bRunning = FALSE;
		ee.Kill();
	}
	CDialog::OnCancel();
}
