// $Id: blender2csDlg.cpp 72 2003-10-24 23:58:51Z felfert $
//
// blender2csDlg.cpp : implementation file
//

#include "stdafx.h"
#include "blender2cs.h"
#include "blender2csDlg.h"
#include "aboutdlg.h"
#include "LogDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Definitions of keys for saving data in Registry
#define RKEY_HISTORY	_T("History")
#define RKEY_SETTINGS	_T("Settings")
#define RKEY_INPUT		_T("Input")
#define RKEY_OUTPUT		_T("Output")
#define RKEY_TEXTURES	_T("TextureDir")
#define RKEY_OUTMODE	_T("OutputMode")
#define RKEY_INPUTDIR	_T("InputDir")
#define RKEY_OUTPUTDIR	_T("OutputDir")
#define RKEY_FLAGS		_T("Flags")

#define FLAGS_LISTFILE 0x01
#define FLAGS_FIXLAMPS 0x02

// Max number of history entries
#define MAXHIST 5

/////////////////////////////////////////////////////////////////////////////
// CBlender2csDlg dialog

CBlender2csDlg::CBlender2csDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CBlender2csDlg::IDD, pParent)
{
	CWinApp *a = AfxGetApp();
	DWORD flags = a->GetProfileInt(RKEY_SETTINGS, RKEY_FLAGS, 0);

	//{{AFX_DATA_INIT(CBlender2csDlg)
	m_sInputFilename = a->GetProfileString(RKEY_HISTORY, RKEY_INPUT, _T(""));
	m_sOutputFilename = a->GetProfileString(RKEY_HISTORY, RKEY_OUTPUT, _T(""));
	m_sTextureDir = a->GetProfileString(RKEY_HISTORY, RKEY_TEXTURES, _T(""));
	m_nOutputMode = a->GetProfileInt(RKEY_SETTINGS, RKEY_OUTMODE, 0);
	m_bListFile = (flags & FLAGS_LISTFILE) ? TRUE : FALSE;
	m_bFixLamps = (flags & FLAGS_FIXLAMPS) ? TRUE : FALSE;
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CBlender2csDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CBlender2csDlg)
	DDX_Control(pDX, IDC_COMBO_TEXDIR, m_cTextureDir);
	DDX_Control(pDX, IDC_COMBO_INPUT, m_cInputFilename);
	DDX_Control(pDX, IDC_COMBO_OUTPUT, m_cOutputFilename);
	DDX_CBString(pDX, IDC_COMBO_INPUT, m_sInputFilename);
	DDX_CBString(pDX, IDC_COMBO_OUTPUT, m_sOutputFilename);
	DDX_CBString(pDX, IDC_COMBO_TEXDIR, m_sTextureDir);
	DDX_Radio(pDX, IDC_RADIO_LEVEL, m_nOutputMode);
	DDX_Check(pDX, IDC_CHECK_LISTFILE, m_bListFile);
	DDX_Check(pDX, IDC_CHECK_FIXLAMPS, m_bFixLamps);
	//}}AFX_DATA_MAP
	GetHistory(RKEY_INPUT, m_cInputFilename);
	GetHistory(RKEY_OUTPUT, m_cOutputFilename);
	GetHistory(RKEY_TEXTURES, m_cTextureDir);
}

BEGIN_MESSAGE_MAP(CBlender2csDlg, CDialog)
	//{{AFX_MSG_MAP(CBlender2csDlg)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BROWSE_OUTPUT, OnBrowseOutput)
	ON_BN_CLICKED(IDC_BROWSE_INPUT, OnBrowseInput)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBlender2csDlg message handlers

BOOL CBlender2csDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CBlender2csDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CBlender2csDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CBlender2csDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CBlender2csDlg::OnBrowseInput() 
{
	CString title, filter;
	CWinApp *a = AfxGetApp();

	title.LoadString(IDS_FDLG_INPNAME);
	filter.LoadString(IDS_FFILT_INPNAME);
	CFileDialog fdlg(TRUE, _T("blend"), m_sInputFilename,
		OFN_NOCHANGEDIR | OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST | OFN_READONLY , filter, this);

#ifdef OFN_DONTADDTORECENT
	fdlg.m_ofn.Flags |= OFN_DONTADDTORECENT;
#endif

	fdlg.m_ofn.lpstrTitle = title;
	CString idir = a->GetProfileString(RKEY_SETTINGS, RKEY_INPUTDIR);
	if (!idir.IsEmpty())
		fdlg.m_ofn.lpstrInitialDir = idir;

	if (fdlg.DoModal() == IDOK) {
		CString path = fdlg.GetPathName();
		m_cInputFilename.SetWindowText(path);
		a->WriteProfileString(RKEY_SETTINGS, RKEY_INPUTDIR, path.Left(path.GetLength() - fdlg.GetFileName().GetLength() - 1));
	}
}

void CBlender2csDlg::OnBrowseOutput() 
{
	CString title, filter;
	CWinApp *a = AfxGetApp();

	title.LoadString(IDS_FDLG_OUTNAME);
	filter.LoadString(IDS_FFILT_OUTNAME);
	CFileDialog fdlg(FALSE, _T(""), m_sOutputFilename,
		OFN_HIDEREADONLY | OFN_NOCHANGEDIR | OFN_PATHMUSTEXIST | OFN_OVERWRITEPROMPT, filter, this);

#ifdef OFN_DONTADDTORECENT
	fdlg.m_ofn.Flags |= OFN_DONTADDTORECENT;
#endif

	CString idir = a->GetProfileString(RKEY_SETTINGS, RKEY_OUTPUTDIR);
	if (!idir.IsEmpty())
		fdlg.m_ofn.lpstrInitialDir = idir;

	fdlg.m_ofn.lpstrTitle = title;
	if (fdlg.DoModal() == IDOK) {
		CString path = fdlg.GetPathName();
		m_cOutputFilename.SetWindowText(path);
		a->WriteProfileString(RKEY_SETTINGS, RKEY_OUTPUTDIR, path.Left(path.GetLength() - fdlg.GetFileName().GetLength() - 1));
	}
}

void CBlender2csDlg::OnOK()
{
	CWinApp *a = AfxGetApp();

	UpdateData();
	if (m_sInputFilename.IsEmpty()) {
		AfxMessageBox(IDS_ERR_NOINPUT);
		return;
	}
	if (m_sOutputFilename.IsEmpty()) {
		AfxMessageBox(IDS_ERR_NOOUTPUT);
		return;
	}
	MergeHistory(RKEY_INPUT, m_sInputFilename, false);
	MergeHistory(RKEY_OUTPUT, m_sOutputFilename, false);
	MergeHistory(RKEY_TEXTURES, m_sTextureDir, true);
	a->WriteProfileInt(RKEY_SETTINGS, RKEY_OUTMODE, m_nOutputMode);
	DWORD flags = 0;
	flags |= m_bListFile ? FLAGS_LISTFILE : 0;
	flags |= m_bFixLamps ? FLAGS_FIXLAMPS : 0;
	a->WriteProfileInt(RKEY_SETTINGS, RKEY_FLAGS, flags);

	LogDlg logger(this);
	logger.DoModal();

	CDialog::OnOK();
}

#define MAXHISTORY 5

void CBlender2csDlg::MergeHistory(CString sSection, CString sValue, bool bCaseSensitive)
{
	CWinApp *a = AfxGetApp();
	CStringArray saHistory;
	int i;

	// Read all history entries
	for (i = 0; i < MAXHISTORY; i++) {
		CString sec;
		sec.Format("%s%d", sSection, i);
		CString tmp = a->GetProfileString(RKEY_HISTORY, sec, _T(""));
		if (!tmp.IsEmpty())
			saHistory.Add(tmp);
	}
	// Check, if we already have sValue in array
	bool found = false;
	for (i = 0; i < saHistory.GetSize(); i++) {
		if (bCaseSensitive) {
			if (sValue.Compare(saHistory[i]) == 0) {
				found = true;
				break;
			}
		} else {
			if (sValue.CompareNoCase(saHistory[i]) == 0) {
				found = true;
				break;
			}
		}
	}
	// Insert sValue, if not already in array
	if (!found)
		saHistory.InsertAt(0, sValue);
	// Write back to Registry
	if (saHistory.GetSize() > MAXHISTORY)
		saHistory.SetSize(MAXHISTORY);
	for (i = 0; i < saHistory.GetSize(); i++) {
		CString sec;
		sec.Format("%s%d", sSection, i);
		a->WriteProfileString(RKEY_HISTORY, sec, saHistory[i]);
	}

	a->WriteProfileString(RKEY_HISTORY, sSection, sValue);
}

void CBlender2csDlg::GetHistory(CString sSection, CComboBox &cCombo)
{
	CWinApp *a = AfxGetApp();
	int i;

	// Read all history entries
	for (i = 0; i < MAXHISTORY; i++) {
		CString sec;
		sec.Format("%s%d", sSection, i);
		CString tmp = a->GetProfileString(RKEY_HISTORY, sec, _T(""));
		if (!tmp.IsEmpty()) {
			cCombo.AddString(tmp);
		}
	}
}
