/********************************************************************
**                                                                 **
**  Blender File          author: celer                            **
**  Library               email: celer@scrypt.net                  **
**                        Copywrite (C) 2000 by David Tyree.       **
**                        All rights reserved.                     **
*********************************************************************
* This library is free software; you can redistribute it and/or     *
* modify it under the terms of the GNU Library General Public       *
* License as published by the Free Software Foundation; either      *
* version 2 of the License, or (at your option) any later version.  *
*                                                                   *
* This library is distributed in the hope that it will be useful,   *
* but WITHOUT ANY WARRANTY; without even the implied warranty of    *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU *
* Library General Public License for more details.                  *
*                                                                   *
* You should have received a copy of the GNU Library General Public *
* License along with this library; if not, write to the Free        *
* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.*
*********************************************************************
$Id: bparse.h 42 2003-06-30 07:46:29Z kludge $
********************************************************************/ 
#ifndef BPARSE_H
#define BPARSE_H

BEGIN_C_DECLS

#ifndef __cplusplus
extern uint8 swap;        /* Should we swap the bytes or not 
                           .. little or big endian */
#endif

/* BLParseFile
 Parse the file specified by the BLFile struct 
	file - file to parse

	returns TRUE upon sucess FALSE otherwise.
*/
int BLParseFile(BLFile *file);


/********************* internal functions *********************************/

/* Read the specified type from the pointer */
uint32 BReadUINT32(char *data);
uint16 BReadUINT16(char *data);
int32 BReadINT32(char *data);
int16 BReadINT16(char *data);

/* Find the specified struct when given the type */
int32 BLFindStrForType(BLFile *file, uint16 type);

/* Print some number of tabs */
void BLPrintTab(uint16 c);

/* When given type and variable name return it's size */
int BLDataSize(BLFile *file, uint16 type, char *varname);

/* When given type and variable name return it's size */
int BLArraySize(BLFile *file, uint16 type, char *varname);

/* Is the specified variable name a pointer */
int BLIsPtr(char *varname);

/* Print the type */
void BLPrintType(BLFile *file, void *ptr, uint16 type, uint16 var);


/* Load a structure */
char *BLGetStruct(BLFile *file, BLStruct *strc, void *strptr, char *pt, uint32 type);

/* Used by BLParseFile to parse all the structures and build them */
void BLDumpData(BLFile *file);

typedef struct _BLPtrList {
  BLStruct *strc;
  struct _BLPtrList *next;
} BLPtrList;

/* Build hash with original struct pointers */
void BLBuildPtrHash(BLFile *file);

/* Free pointer hash */
void BLFreePtrHash();

/* When give the blender pointer find the real one */
BLStruct *BLFindPtr(void *ptr);

/* Resolve as many pointers as we can */
int BLResolvePtrs(BLStruct *strc);

END_C_DECLS

#endif
