/********************************************************************
**                                                                 **
**  Blender File          author: celer                            **
**  Library               email: celer@scrypt.net                  **
**                        Copywrite (C) 2000 by David Tyree.       **
**                        All rights reserved.                     **
*********************************************************************
* This library is free software; you can redistribute it and/or     *
* modify it under the terms of the GNU Library General Public       *
* License as published by the Free Software Foundation; either      *
* version 2 of the License, or (at your option) any later version.  *
*                                                                   *
* This library is distributed in the hope that it will be useful,   *
* but WITHOUT ANY WARRANTY; without even the implied warranty of    *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU *
* Library General Public License for more details.                  *
*                                                                   *
* You should have received a copy of the GNU Library General Public *
* License along with this library; if not, write to the Free        *
* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.*
*********************************************************************
$Id: bdata.c 11 2003-01-05 22:27:27Z gmueckl $
********************************************************************/ 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "btypes.h"
#include "bfile.h"
#include "bdata.h"
#include "bparse.h"

/* Find the struct with the specified name */
BLStruct *BLFindStruct(BLStruct *head,const char *name){
	BLStruct *mStruct;
	for(mStruct=head;mStruct!=NULL;mStruct=mStruct->next){
		if(strcmp(mStruct->struct_name,name)==0)
				return mStruct;
	}
	return NULL;
}



/* specify the path to the requested data */
/* ex  id.newid      */
BLData *BLFindData(BLStruct *strc,const char *path){
	char *mPath=strdup(path);
	char *mElement=mPath;
	BLData *mData=NULL;
	uint32 i;
	
	while((mElement=strsep(&mPath,"."))){
		for(i=0;i<strc->length;i++){
			char *mVar=strc->data[i]->var_name;
			if(mVar[0]=='*') mVar++;
			if(strncmp(mVar,mElement,strlen(mElement))==0){
				if(strc->data[i]->type!=BSTRC)
					mData=strc->data[i];
				else 
				if(strchr(path,'.')){
					mData=BLFindData(
									(BLStruct *) strc->data[i]->data.strc,
									strchr(path,'.')+1);
				}
				break;
			}
		}	
	}	
	free(mPath);
	return mData;
}

void BLPrintData(BLData *data,uint16 level){
	uint16 i;
	char *sep="";
	if(data->type!=BSTRC){
		BLPrintTab(level);
		printf("%s %s %u %u = ",data->type_name,data->var_name,data->length,data->type);
	}
	if(data->length>1 && data->type!=BCHR){ printf("{"); sep=","; }
	for(i=0;i<data->length;i++){
		switch(data->type){
			case BFLOAT:
				printf(" %f%s",*data->data.flt +i,sep);
			break;
			case BINT:
				printf(" %d%s",*data->data.intg +i,sep );
			break;
			case BSHORT:
				printf(" %d%s",*data->data.shrt +i,sep );
			break;
			case BCHR:
				if(data->length>1){
						if(i==1)
							printf("\"%s\"",data->data.chr);
				}
				else
					printf("0x%x%s",*data->data.chr+i,sep);
			break;
			case BPTR:
				printf(" 0x%x%s",BReadUINT32((char *)(data->data.ptr +i)),sep );
			break;
			case BSTRC:
				BLPrintStruct(data->data.strc,level);
				printf(" %s",data->var_name);
			break;
		}
	}
	if(data->length>1 && data->type!=BCHR ) printf("}");
	printf(";\n");
}

void BLPrintStructs(BLFile *file){
	BLStruct *mTemp;
	for(mTemp=file->struct_head;mTemp!=NULL;mTemp=mTemp->next){
		BLPrintStruct(mTemp,0);
		printf(";\n");
	}

}


void BLPrintStruct(BLStruct *strc,uint16 level){
    uint16 i;
    BLPrintTab(level);
    printf("struct %s { // %x \n",strc->struct_name,strc);
    for(i=0;i<strc->length;i++){
        BLPrintData(strc->data[i],level+1);
    }
    BLPrintTab(level);
    printf("}");
}

BLData *BLNewData(void){
	BLData *mData;
	mData=(BLData *) malloc(sizeof(BLData));
	memset(mData,0,sizeof(BLData));
	return mData;
}

BLData *BLInitDataFloat(char *var,char *type,float *flt,uint16 len){
	BLData *mData=BLNewData();

	mData->var_name=var;
	mData->type_name=type;
	mData->data.flt=flt;
	mData->type=BFLOAT;
	mData->length=len;

	return mData;
}

BLData *BLInitDataChar(char *var,char *type,char *chr,uint16 len){
	BLData *mData=BLNewData();

	mData->var_name=var;
	mData->type_name=type;
	mData->data.chr=chr;
	mData->type=BCHR;
	mData->length=len;

	return mData;
}


BLData *BLInitDataInt(char *var,char *type,uint32 *intg,uint16 len){
	BLData *mData=BLNewData();

	mData->var_name=var;
	mData->type_name=type;
	mData->data.intg=intg;
	mData->type=BINT;
	mData->length=len;

	return mData;
}

BLData *BLInitDataShort(char *var,char *type,short *shrt,uint16 len){
	BLData *mData=BLNewData();

	mData->var_name=var;
	mData->type_name=type;
	mData->data.shrt=shrt;
	mData->type=BSHORT;
	mData->length=len;

	return mData;
}

BLData *BLInitDataPointer(char *var,char *type,void *ptr,uint16 len){
	BLData *mData=BLNewData();

	mData->var_name=var;
	mData->type_name=type;
	mData->data.ptr=ptr;
	mData->type=BPTR;
	mData->length=len;

	return mData;
}



BLData *BLInitDataStruct(char *var,char *type,BLStruct *ptr,uint16 len){
	BLData *mData=BLNewData();

	mData->var_name=var;
	mData->type_name=type;
	mData->data.strc=ptr;
	mData->type=BSTRC;
	mData->length=len;
	
	return mData;
}

BLStruct *BLNewStruct(){
	BLStruct *mData;
	mData=(BLStruct *) malloc(sizeof(BLStruct));
	return mData;
}

BLStruct *BLInitStruct(char *name,void *ptr,uint16 length){
	BLStruct *mData=BLNewStruct();

	mData->struct_name=name;
	mData->ptr=ptr;
	mData->length=length;
	mData->data=malloc(sizeof(BLData *)*length);
	mData->next=NULL;
	mData->prev=NULL;
	return mData;
}

void BLFreeData(BLData *data){
	free(data);
}

void BLFreeStruct(BLStruct *data){
	free(data->data);
	free(data);
}

void BLFreeStructs(BLStruct *head){
	BLStruct *temp,*tempNext=head;
	uint32 i;
	for(temp=head;temp!=NULL;temp=temp->next){
		for(i=0;i<temp->length;i++)
			BLFreeData(temp->data[i]);
	}
	temp=head;
	while(temp){
		tempNext=temp->next;
		BLFreeStruct(temp);
		temp=tempNext;
	}
}




