/********************************************************************
**                                                                 **
**  Blender File          author: celer                            **
**  Library               email: celer@scrypt.net                  **
**                        Copywrite (C) 2000 by David Tyree.       **
**                        All rights reserved.                     **
*********************************************************************
* This library is free software; you can redistribute it and/or     *
* modify it under the terms of the GNU Library General Public       *
* License as published by the Free Software Foundation; either      *
* version 2 of the License, or (at your option) any later version.  *
*                                                                   *
* This library is distributed in the hope that it will be useful,   *
* but WITHOUT ANY WARRANTY; without even the implied warranty of    *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU *
* Library General Public License for more details.                  *
*                                                                   *
* You should have received a copy of the GNU Library General Public *
* License along with this library; if not, write to the Free        *
* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.*
*********************************************************************
$Id: bfile.h 6 2003-01-04 16:04:59Z gmueckl $
********************************************************************/ 
#ifndef BFILE_H
#define BFILE_H

BEGIN_C_DECLS

/* The root of all evil.... 
	
	These data structures are mainly used during parsing to deal
	with the internal structure of the blender file format. They
	are of little interest to most people. 
*/


/* Don't ask me why... 
	This structure is a place holder for most of the interesting
 	data that we find in the file
*/
typedef struct {
	uint32 var_len;		/* The number of variable names */
	char **var_name;	/* ptr to an array of variable names */
	
	uint32 type_len;	/* The number of types */
	char **type_name;	/* ptr to an array of type names */
	uint16 *type_size;	/* array of types sizes corresponding to names */
	
	uint32 strc_len;	/* The number of structures */
	uint16 **strc_data;	/* ptr to structure data */
} DNA;

struct _BLStruct;


/* 
	This is our main container for all the data. 
	It contains pointers to interesting things in the
	file along with our final parsed output.
*/
typedef struct {
	FILE *file;		/* Pointer to the file handle */
	uint32 file_size;	/* Size of the file in bytes */
	uint16 version;		/* Version of the blender file */	
	
	DNA dna;		/* The above DNA struct */

	char *rend_data;	/* ptr to render data */
	uint32 rend_len;	/* length of render data */
	
	char *dna_data;		/* ptr to dna data */
	uint32 dna_len;		/* length of dna data */
		
	char *sdna_data;	/* ptr to sdna data */
	uint32 sdna_len;	/* lenght of sdna data */
	
	char *name_data;	/* ptr to name data */
	uint32 name_len;	/* name data length */
	
	char *type_data;	/* ptr to type data */
	uint32 type_len;	/* type data length */
	
	char *tlen_data;	/* ptr to type length data */
	uint32 tlen_len;	/* type length data length */
			
	char *strc_data;	/* ptr to structure data */
	uint32 strc_len;	/* structure data length */

	/* This contains the actual parsed struct data */
	struct _BLStruct *struct_head;
	
} BLFile;

/* BLOpenFile 
 Open the specified file
	name - name of the file to open 	

	returns NULL upon failure
*/
BLFile *BLOpenFile(const char *name);


/* BLClosefile 
 Close the specified file structure 
	file - ptr to file structure to close 
*/
void BLCloseFile(BLFile *file);

END_C_DECLS

#endif
