/********************************************************************
**                                                                 **
**  Blender File          author: celer                            **
**  Library               email: celer@scrypt.net                  **
**                        Copywrite (C) 2000 by David Tyree.       **
**                        All rights reserved.                     **
*********************************************************************
* This library is free software; you can redistribute it and/or     *
* modify it under the terms of the GNU Library General Public       *
* License as published by the Free Software Foundation; either      *
* version 2 of the License, or (at your option) any later version.  *
*                                                                   *
* This library is distributed in the hope that it will be useful,   *
* but WITHOUT ANY WARRANTY; without even the implied warranty of    *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU *
* Library General Public License for more details.                  *
*                                                                   *
* You should have received a copy of the GNU Library General Public *
* License along with this library; if not, write to the Free        *
* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.*
*********************************************************************
$Id: bparse.c 39 2003-06-30 05:43:27Z kludge $
********************************************************************/ 
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include "btypes.h"
#include "bfile.h"
#include "bdata.h"
#include "butil.h"
#include "bparse.h"

#define BLENDER_MAGIC "BLENDER"
#define REND_MAGIC "REND"
#define DNA_MAGIC "DNA1"
#define SDNA_MAGIC "SDNA"
#define NAME_MAGIC "NAME"
#define TYPE_MAGIC "TYPE"
#define TLEN_MAGIC "TLEN"
#define STRC_MAGIC "STRC"
#define ENDB_MAGIC "ENDB"

uint8 swap=FALSE;

uint32 BReadUINT32(char *data){
  if(swap)
    return swap4(*((uint32 *) data));
  else return *((uint32 *) data);
}

uint16 BReadUINT16(char *data){
  if(swap)
    return swap2(*((uint16 *) data));
  else return *((uint16 *) data);
}

int32 BReadINT32(char *data){
  if(swap)
    return swap4(*((int32 *) data));
  else return *((int32 *) data);
}

int16 BReadINT16(char *data){
  if(swap)
    return swap2(*((int16 *) data));
  else return *((int16 *) data);
}

static void *Start_of_data, *End_of_data;
int BLValidPointer(void *p)
{ // I'm using this routine to determine if pointers that we resolve are valid.
  if (p> End_of_data || p < Start_of_data) return 0;
  return 1;
}


int BLParseFile(BLFile *file){
  char blenderMagic[13];
  char *blenderData;
  uint32 mLoc;
  uint32 mBytesRead;  
  uint32 mStrs;
  char *mTempStr;
  uint16 *mTempInt;
  uint32   mREND=0, mDNA=0, 
      mSDNA=0, mNAME=0, 
      mTYPE=0, mTLEN=0, 
      mSTRC=0, mENDB=0;
  uint32 byteOrder;

  // This is the same code that Blender itself uses to determine endianess of current platform
        byteOrder= 1;
        byteOrder= ((char*)&byteOrder)[0];

  /* First lets read in the blender version/magic info */
  memset(blenderMagic,0,13);
  fread(blenderMagic,sizeof(char),12,file->file);
#ifdef DEBUG
  printf("blenderMagic = \"%s\"\n",blenderMagic);
#endif  
  
  if(strncasecmp(blenderMagic,BLENDER_MAGIC,7)!=0)
    return FALSE;

  if(blenderMagic[7]=='-') {
    printf("file format is 32 bit ");
  } else {
    printf("file format is 64 bit ");
  }

  if(blenderMagic[8]=='V') {
    printf("big endian\n");
    if(byteOrder==1) {
      swap=TRUE;
    }
  } else {
    printf("little endian\n");
    if(byteOrder==0) {
      swap=TRUE;
    }
  }

  if(swap==TRUE) {
    printf("byte-swapping data in input file\n");
  }

#if DEBUG
  printf("blenderVersion = %d\n",atoi(blenderMagic+9));
#endif 
  file->version=atoi(blenderMagic+9);

  /* Now that we know were dealing with a blender file, lets 
   * get info about the sections */
#if DEBUG
  printf("file size = %d\n",file->file_size);
#endif 
  blenderData=(char *) malloc(sizeof(char)*file->file_size);  
  if(!blenderData)
      return FALSE;
  
  mBytesRead=fread(blenderData,sizeof(char),file->file_size,file->file);
  Start_of_data=blenderData;
  End_of_data=Start_of_data+mBytesRead;


  /* Ok now that we have loaded this honking think into memory we need to
   * chunk it off into seperate sections so that it *could* be modified later
   */

#define CMP_MAGIC(a,b) if(0==a && strncmp(blenderData+mLoc,b,4)==0)\
                a=mLoc+4;
  
  for(mLoc=0;mLoc<mBytesRead;mLoc++){  
    CMP_MAGIC(mREND,REND_MAGIC);
    CMP_MAGIC(mDNA,DNA_MAGIC);
    CMP_MAGIC(mSDNA,SDNA_MAGIC);
    CMP_MAGIC(mNAME,NAME_MAGIC);
    CMP_MAGIC(mTYPE,TYPE_MAGIC);
    CMP_MAGIC(mTLEN,TLEN_MAGIC);
    CMP_MAGIC(mSTRC,STRC_MAGIC);  
    CMP_MAGIC(mENDB,ENDB_MAGIC);
  }
#if DEBUG 
  printf("REND = %d\n", mREND);
  printf("DNA = %d\n", mDNA);
  printf("SDNA = %d\n", mSDNA);
  printf("NAME = %d\n", mNAME);
  printf("TYPE = %d\n", mTYPE);
  printf("TLEN = %d\n", mTLEN);
  printf("STRC = %d\n", mSTRC);
  printf("ENDB = %d\n", mENDB);
#endif


#define GET_SECTION(a,b,c,d)\
    c=blenderData+a; \
    d=b-a;
    
  GET_SECTION(mREND,mDNA,file->rend_data,file->rend_len);
  GET_SECTION(mDNA,mSDNA,file->dna_data,file->dna_len);
  GET_SECTION(mSDNA,mNAME,file->sdna_data,file->sdna_len);
  GET_SECTION(mNAME,mTYPE,file->name_data,file->name_len);
  GET_SECTION(mTYPE,mTLEN,file->type_data,file->type_len);
  GET_SECTION(mTLEN,mSTRC,file->tlen_data,file->tlen_len);
  GET_SECTION(mSTRC,mENDB,file->strc_data,file->strc_len);


  /* Time to parse the Name section */
  file->dna.var_len=BReadUINT32(file->name_data);
#if DEBUG
  printf("Number of variables = %d(%x)\n",file->dna.var_len,file->dna.var_len);
#endif
  file->dna.var_name=(char **) malloc(sizeof(char *) *file->dna.var_len);
  mTempStr=file->name_data+4;
  for(mStrs=0;mStrs<file->dna.var_len;mStrs++){
#if DEBUG 
    printf("var_name[%d]=\"%s\"\n",mStrs,mTempStr);
#endif    
    file->dna.var_name[mStrs]=mTempStr;
    mTempStr=strchr(mTempStr,'\0')+1;
  }

  /* Time to parse the type sections */
  file->dna.type_len=BReadUINT32(file->type_data);
  file->dna.type_name=(char **) malloc(sizeof(char *) *file->dna.type_len);
  file->dna.type_size=(uint16 *) malloc(sizeof(uint16 *) *file->dna.type_len);
  mTempStr=file->type_data+4;
  for(mStrs=0;mStrs<file->dna.type_len;mStrs++){
#if DEBUG
    printf("type_name[%d]=\"%s\"(%d)\n",mStrs,mTempStr,
        BReadUINT16(file->tlen_data+2*mStrs)
        );

#endif
    file->dna.type_name[mStrs]=mTempStr;
    file->dna.type_size[mStrs]=BReadUINT16(file->tlen_data+2*mStrs);
    mTempStr=strchr(mTempStr,'\0')+1;
  }


  file->dna.strc_len=BReadUINT32(file->strc_data);
  file->dna.strc_data=(uint16 **)  malloc(sizeof(uint16 **) *file->dna.strc_len);

  #if DEBUG
    printf("Number of structures =%d(%x)\n",file->dna.strc_len,file->dna.strc_len);
  #endif

  mTempInt=(uint16 *)(file->strc_data+4);
  for(mLoc=0;mLoc<file->dna.strc_len;mLoc++)
  {
    uint16 mStrcSize,mStrcType;
    mStrcType=BReadUINT16((char *) mTempInt);
    mStrcSize=BReadUINT16((char *) (mTempInt+1));
  
    #if DEBUG
    {
      uint16 mStrcIndex;    
      printf("%d struct %s\n{\n",mLoc,file->dna.type_name[mStrcType]);
      for(mStrcIndex=0;mStrcIndex<mStrcSize;mStrcIndex++)
      {
        printf("\t %s",file->dna.type_name[BReadUINT16((char*)(mTempInt+(mStrcIndex*2)+2))]);
        printf(" %s;\n",file->dna.var_name[BReadUINT16((char*)(mTempInt+(mStrcIndex*2)+3))]);
      }
      printf("};\n");//,(mStrcSize));
    }
    #endif    

    /* fill in the structer data */
    file->dna.strc_data[mLoc]=mTempInt;
    mTempInt+=(mStrcSize*2)+2;
  }
  BLDumpData(file);  
  //FIXME return failure or success
  return 0;
  //
}


int32 BLFindStrForType(BLFile *file, uint16 type){
  uint16 i;
  for(i=0;i<file->dna.strc_len;i++){
    if(file->dna.strc_data[i][0]==type)
      return i;
  }
  return -99;
}

void BLPrintTab(uint16 c){
  while(c>0){
    printf("  ");
    c--;
  }
}


int BLDataSize(BLFile *file,uint16 type,char *varname){
        if(varname==NULL)
        {
          printf("BLDataSize(%p,%d,NULL) called.\n",file,type);
          return 0;
        }
  /* Pointers are usually 4 */
  if(strchr(varname,'*')) return 4;
  /* Oh shoot it is an array. We mist figure out how big it is. */
  if(strchr(varname,'[')){
    uint32 mSize=file->dna.type_size[type];
    char *mTempStr=strdup(varname);
    char *mToken;
    mTempStr=strchr(mTempStr,'[')+1;
    while((mToken=strsep(&mTempStr,"[]"))){
      if(strlen(mToken)>0)
        mSize=mSize*atoi(mToken);
    }
    free(mTempStr);
    return mSize;

  }
  return file->dna.type_size[type];
}

int BLArraySize(BLFile *file,uint16 type,char *varname){
  /* Pointers are usually 4 */
  if(strchr(varname,'*')) return 1;
  /* Oh shoot it is an array. We mist figure out how big it is. */
  if(strchr(varname,'[')){
    uint32 mSize=1;
    char *mTempStr=strdup(varname);
    char *mToken;
    mTempStr=strchr(mTempStr,'[')+1;
    while((mToken=strsep(&mTempStr,"[]"))){
      if(strlen(mToken)>0)
        mSize=mSize*atoi(mToken);
    }
    free(mTempStr);
    return mSize;

  }
  return 1;
}

int BLIsPtr(char *varname){
  if(strchr(varname,'*')) return TRUE;
  return FALSE;
}

void BLPrintType(BLFile *file, void *ptr,uint16 type,uint16 var){
  char *mType, *mVar;

  mType=file->dna.type_name[type];
  mVar=file->dna.var_name[var];
  
  if(BLIsPtr(mVar)) printf("%x",BReadUINT32(ptr));
  
  
}



char *BLGetStruct(BLFile *file, BLStruct *strc,void *strptr, char *pt, uint32 type)
{
// BLStruct *mStruct; // unused here
  uint32 i=0;
  uint16 *mTempInt;

  /* Set up the struct data  structure */
  //  mTempInt=(uint16 *)(file->strc_data+4);
  mTempInt=(uint16 *)&(file->dna.strc_data[type][0]);
  strc->struct_name=file->dna.type_name[BReadUINT16((char *) mTempInt)];

#ifdef DEBUG
  printf("-mTempInt: %p\n",mTempInt);
  printf("-pt: %p\n",pt);
  printf("struct_name: %s\n",strc->struct_name);
#endif

  strc->data=(BLData **) malloc(sizeof(BLData *) * BReadUINT16((char *)(mTempInt+1)));
  strc->ptr=strptr;

  strc->length=BReadUINT16((char *)(mTempInt+1));
#ifdef DEBUG
  printf("max: %d\n",strc->length);
#endif

  for(i=0;i<strc->length;i++)
  {  
#ifdef DEBUG
    printf("i: %d\n",i);
#endif
    //if(BLFindStrForType(file,file->dna.strc_data[type][i*2+2])!=-99 && 
    if(BLFindStrForType(file,BReadUINT16((char *)(mTempInt+(i*2)+2)))!=-99 && file->dna.var_name[BReadUINT16((char *)(mTempInt+(i*2)+3))][0]!='*')
    {
      uint32 mType=BLFindStrForType(file,BReadUINT16((char *)(mTempInt+(i*2)+2)));
                                       //file->dna.strc_data[type][i*2+2]);

      BLStruct *mStruct=BLNewStruct();
#ifdef DEBUG
      printf("mType: %d(%x)\n",mType,mType);
      printf("recursive BLGetStruct(%p,%p,NULL,%p,%i)\n",file,mStruct,pt,mType);    
#endif
      pt=BLGetStruct(file,mStruct,NULL,pt,mType);

      strc->data[i]=BLInitDataStruct(file->dna.var_name[BReadUINT16((char *)(mTempInt+(i*2)+3))], 
                                                       // file->dna.strc_data[type][i*2+3]],
              file->dna.type_name[BReadUINT16((char *)(mTempInt+(i*2)+2))],
                                     //file->dna.strc_data[type][i*2+2]],
              mStruct,1);
    }
    else 
    {
      char *mType=NULL, *mVar=NULL;
      uint16 mSize=0,mLength=0;

      uint16 val=0;

      val=BReadUINT16((char *)((mTempInt+(i*2)+2)));
      //printf("type_name[%d]=",val);
      mType=file->dna.type_name[val];
#ifdef DEBUG
      printf("%s\n",mType);
#endif

      val=BReadUINT16((char *)((mTempInt+(i*2)+3)));
      //printf("var_name[%d]=",val);
      mVar=file->dna.var_name[val];
#ifdef DEBUG
      printf("%s\n",mVar);
#endif

      val=BReadUINT16((char *)((mTempInt+(i*2)+2)));
      //printf("data_size[%d]=",val);
      mSize=BLDataSize(file,val,mVar);
#ifdef DEBUG
      printf("%d\n",mSize);
#endif

      val=BReadUINT16((char *)((mTempInt+(i*2)+2)));
      //printf("array_size[%d]=",val);
      mLength=BLArraySize(file,val,mVar);
#ifdef DEBUG
      printf("%d\n",mLength);
#endif
      
      if(strcmp(mType,"float")==0)
        strc->data[i]=BLInitDataFloat(mVar,mType,(float *) pt,mLength);
      else if(strcmp(mType,"int")==0)
        strc->data[i]=BLInitDataInt(mVar,mType,(uint32 *) pt,mLength);
      else if(strcmp(mType,"short")==0 || strcmp(mType,"ushort")==0)
        strc->data[i]=BLInitDataShort(mVar,mType,(short *) pt,mLength);
      else if(strcmp(mType,"char")==0 || strcmp(mType,"uchar")==0)
        strc->data[i]=BLInitDataChar(mVar,mType,(char *) pt,mLength);
      else {
	//	printf("ZZZZ %x %s\n",pt,pt);
	strc->data[i]=BLInitDataPointer(mVar,mType,(void *) pt,mLength);
      }
      
      pt+=mSize;
    }
  }
  return pt;
}


void BLDumpData(BLFile *file)
{
  BLStruct *mTempStruct=NULL;
  uint32 mSize=0;
  void  *mPointer=NULL;
  uint32 mType=0;
  uint32 mNumber=0;
  uint32 mTotalNumber=0;
//  char *mName; // unused
  char *mTempPtr=file->rend_data;
//  uint32 count; // unused
  BLStruct *strc=NULL;
  int i=0;
  char *mTempPtr2=NULL;
  
  /* Skip the info about the render data */
  mSize=BReadUINT32(mTempPtr);  
//  mPointer=(void *) BReadUINT32(mTempPtr+4);  
  mPointer=(void *) mTempPtr+4;  

  mType=BReadUINT32(mTempPtr+8);
  mNumber=BReadUINT32(mTempPtr+12);

#ifdef DEBUG
    printf("mSize=%d\n",mSize);
    printf("mPointer=%p\n",mPointer);
    printf("mType=%d %x\n",mType,mType);
    printf("mNumber=%d\n",mNumber);
#endif

  mTempPtr+=mSize+16;
  mTotalNumber=mSize;


#ifdef DEBUG
    printf("DNA_MAGIC: %s\n",DNA_MAGIC);
    printf("First mTempPtr(%p): %s\n",mTempPtr,mTempPtr);
#endif

  while(strncmp(mTempPtr,DNA_MAGIC,3)!=0)
  {
    mSize=BReadUINT32(mTempPtr+4);  
    mPointer=(void *) (mTempPtr+8);
    mType=BReadUINT32(mTempPtr+12);
    mNumber=BReadUINT32(mTempPtr+16);

#ifdef DEBUG
    printf("mSize=%d\n",mSize);
    printf("mPointer=%p\n",mPointer);
    printf("mType=%d %x\n",mType,mType);
    printf("mNumber=%d\n",mNumber);
#endif

    mTempPtr2=mTempPtr+20;
    for(i=0;i<mNumber;i++) 
    {
      strc=BLNewStruct();
#ifdef DEBUG
      printf("i: %d, strc: %p, file: %p, mPointer: %p, mTempPtr2: %p, mType: %d\n",i,strc,file,mPointer,mTempPtr2,mType);
      printf("direct BLGetStruct(%p,%p,%p,%p,%i)\n",file,strc,mPointer,mTempPtr2,mType);    
#endif


      mTempPtr2=BLGetStruct(file,strc,mPointer,mTempPtr2,mType);

      if(!file->struct_head)
      {
        file->struct_head=strc;
        mTempStruct=strc;
      }
      else 
      {
        mTempStruct->next=strc;
        strc->prev=mTempStruct;
        mTempStruct=strc;
      }
    }
    mTempPtr+=mSize+20;    
#ifdef DEBUG
      printf("Next mTempPtr: %p\n",mTempPtr);
      printf("Next mTempPtr2: %p\n",mTempPtr2);

#endif


  }
  /* Time to resolve as many pointers as we can */
  BLBuildPtrHash(file);
  for(strc=file->struct_head;strc!=NULL;strc=strc->next){
    BLResolvePtrs(strc);
  }
  BLFreePtrHash();
}

BLPtrList *BLPtrHash[65536];

void BLBuildPtrHash(BLFile *file)
{
  BLStruct *strc;
  int i, idx;
  BLPtrList *ptr, *ptr2;

  for(i=0;i<65536;i++) {
    BLPtrHash[i]=NULL;
  }

  for(strc=file->struct_head;strc!=NULL;strc=strc->next) {
    idx=BReadUINT32((char *)strc->ptr) & 0xFFFF; /* Hash the last 2 bytes of the original pointers */
    ptr=malloc(sizeof(BLPtrList)); /* XXX no error checking done */
    ptr->strc=strc;
    if(BLPtrHash[idx]==NULL) {
      BLPtrHash[idx]=ptr;
      ptr->next=NULL;
    } else {
      /* add ptr at end of list in hash slot */
      for(ptr2=BLPtrHash[idx];ptr2->next!=NULL;ptr2=ptr2->next);
      ptr2->next=ptr;
      ptr->next=NULL;
    }
  }
}

void BLFreePtrHash()
{
  int i;
  BLPtrList *temp, *temp2;

  for(i=0;i<65536;i++) {
    if(BLPtrHash[i]!=NULL) {
      for(temp=BLPtrHash[i]->next,temp2=BLPtrHash[i];temp!=NULL;temp=temp->next) {
	free(temp2);
	temp2=temp;
      }
    }
  }
}

BLStruct *BLFindPtr(void *ptr) 
{
  BLPtrList *mTemp;
  int idx;

  if(ptr==NULL) return NULL;

  idx=((int)ptr)&0xFFFF;
  for(mTemp=BLPtrHash[idx];mTemp!=NULL;mTemp=mTemp->next) {
    
    if(ptr==BReadUINT32((char *)mTemp->strc->ptr)) {
#ifdef DEBUG
      printf("found match: %d==%d\n",ptr,BReadUINT32((char *)mTemp->strc->ptr));
#endif
      return mTemp->strc;
    }
  }
  return (BLStruct *) 0xFFFFFFFF;
}

int BLResolvePtrs(BLStruct *strc){
  uint32 i;
  for(i=0;i<strc->length;i++){
    if(strc->data[i]->type==BPTR){
      BLStruct *mStrc;

      mStrc=BLFindPtr((void *) BReadUINT32((char *) strc->data[i]->data.ptr));

      strc->data[i]->stringPtr=NULL;
      if (mStrc==0xFFFFFFFF) {  // Hmm. Didn't find an object
	
	//if (BLValidPointer(BReadUINT32((char *)strc->data[i]->data.ptr))) {
	if (BLValidPointer(strc->data[i]->data.ptr)) {
	  //Copy the string data.
	  strc->data[i]->stringPtr=strndup(strc->data[i]->data.ptr,256);
	}
      }

      *(strc->data[i]->data.ptr)=(uint32 ) mStrc;  
    } else 
    if(strc->data[i]->type==BSTRC)
      BLResolvePtrs(strc->data[i]->data.strc);
  }
//FIXME return failure or success
return 0;
//
}
